"""
Agent - 自主智能体基类

位置: loom/agent/core.py

基于公理系统和唯一性原则：
将所有智能体能力统一到一个Agent类中，作为所有智能体的基础。

设计原则：
1. 唯一性 - 每个功能只在一个地方实现
2. 继承BaseNode - 获得观测和集体记忆能力
3. 集成LLM - 支持流式输出
4. 四范式自动能力 - LLM自主决策使用反思、工具、规划、协作能力

基础能力（继承自BaseNode）：
- 生命周期管理
- 事件发布（观测能力）
- 事件查询（集体记忆能力）
- 统计信息

自主能力（公理A6 - 四范式工作公理）：
- 反思能力：持续的思考过程（通过LLM streaming自动体现）
- 工具使用：LLM自动决策调用工具（通过tool calling）
- 规划能力：LLM检测复杂任务自动规划（通过meta-tool）
- 协作能力：LLM检测需要协作自动委派（通过meta-tool）
"""

import json
import logging
import re
from collections import defaultdict, deque
from pathlib import Path
from typing import TYPE_CHECKING, Any
from uuid import uuid4

from loom.agent.base import BaseNode
from loom.agent.delegator import DelegatorMixin
from loom.agent.executor import ExecutorMixin
from loom.agent.planner import PlannerMixin
from loom.agent.skill_handler import SkillHandlerMixin
from loom.agent.tool_handler import ToolHandlerMixin
from loom.config.context import ContextConfig
from loom.config.memory import MemoryConfig
from loom.config.tool import ToolConfig
from loom.context import ContextOrchestrator
from loom.context.sources import L2WorkingSource, L3PersistentSource
from loom.exceptions import TaskComplete
from loom.memory.compaction import CompactionConfig, MemoryCompactor
from loom.memory.manager import MemoryManager
from loom.memory.segment_store import SegmentStore
from loom.memory.tokenizer import TiktokenCounter
from loom.observability.metrics import LoomMetrics
from loom.observability.tracing import LoomTracer
from loom.providers.llm.interface import LLMProvider
from loom.runtime import Task, TaskStatus
from loom.runtime.session_lane import SessionIsolationMode, SessionLaneInterceptor
from loom.security.tool_policy import ToolPolicy
from loom.tools.builtin.creation import (
    DynamicToolExecutor,
    create_tool_creation_tool,
)
from loom.tools.builtin.done import create_done_tool

# Optional import for SandboxToolManager
try:
    from loom.tools.core.sandbox_manager import SandboxToolManager
except ImportError:
    SandboxToolManager = None  # type: ignore

# Type checking imports (avoid circular imports)
if TYPE_CHECKING:
    from loom.agent.agent_node import NodeType
    from loom.config.context import BudgetConfig
    from loom.events.session import Session
    from loom.memory.shared_pool import SharedMemoryPool
    from loom.tools.skills.activator import SkillActivator


# AgentBuilder 已提取到 loom/agent/builder.py
from loom.agent.builder import AgentBuilder  # noqa: F401
from loom.agent.execution import ExecutionEngine
from loom.agent.factory import AgentFactory
from loom.agent.tool_router import ToolRouter

# Importance tag regex — matches <imp:0.0/> through <imp:1.0/> at end of response
_IMP_TAG_RE = re.compile(r"<imp:((?:0\.\d+|1\.0))/>$")


MEMORY_GUIDANCE = """
<memory_management>
  <search>
    使用 query 工具搜索信息。scope 参数控制搜索范围：
    - scope="auto": 自动搜索所有可用源（默认）
    - scope="memory": 仅搜索对话记忆
    - scope="knowledge": 仅搜索知识库
  </search>

  <importance>
    在回复末尾用 <imp:X.X/> 标记本轮交互的重要性（0.0-1.0）。
    框架会自动处理，用户不会看到这个标记。

    标记时机：
    - 搜索结果对任务有重要参考价值时
    - 做出了关键决策或结论时
    - 用户明确要求记住某些信息时

    不需要标记：
    - 闲聊、确认、简单问答
    - 搜索结果不相关或未被采用时

    示例：
    "建议使用 OAuth2.0 Authorization Code 模式。<imp:0.8/>"
  </importance>
</memory_management>
"""


def _strip_importance_tag(response_text: str) -> tuple[str, float | None]:
    """
    从 Agent 响应中剥离 importance 标记。

    Returns:
        (clean_text, importance) — importance 为 None 表示无标记
    """
    match = _IMP_TAG_RE.search(response_text.rstrip())
    if not match:
        return response_text, None
    importance = float(match.group(1))
    clean_text = response_text[: match.start()].rstrip()
    return clean_text, importance


class Agent(
    ToolHandlerMixin,
    SkillHandlerMixin,
    PlannerMixin,
    DelegatorMixin,
    ExecutorMixin,
    BaseNode,
):
    """
    统一的智能体基类

    继承自BaseNode和多个Mixin，集成了观测、记忆、上下文管理等所有基础能力。

    Mixin职责：
    - ToolHandlerMixin: 工具管理
    - SkillHandlerMixin: 技能管理
    - PlannerMixin: 规划逻辑
    - DelegatorMixin: 委派逻辑
    - ExecutorMixin: 执行循环辅助
    """

    def __init__(
        self,
        node_id: str,
        llm_provider: LLMProvider,
        system_prompt: str = "",  # User's business logic prompt
        tools: list[dict[str, Any]] | None = None,
        available_agents: dict[str, Any] | None = None,
        event_bus: Any | None = None,  # EventBus
        session: "Session | None" = None,  # Session（注入记忆，Agent 变为无状态执行器）
        enable_observation: bool = True,
        max_context_tokens: int = 4000,
        max_iterations: int = 10,
        require_done_tool: bool = True,
        recursive_depth: int = 0,  # 当前递归深度
        config: Any | None = None,  # AgentConfig（Phase 3: 12.5.2）
        skill_registry: Any | None = None,  # SkillRegistry
        skill_activator: Any | None = None,  # SkillActivator（Phase 2）
        tool_registry: Any | None = None,  # ToolRegistry
        sandbox_manager: "SandboxToolManager | None" = None,  # 沙盒工具管理器（NEW）
        parent_memory: "MemoryManager | None" = None,  # 父节点的 MemoryManager
        root_context_id: str | None = None,
        memory_config: dict[str, Any] | None = None,
        context_budget_config: "BudgetConfig | dict[str, float | int] | None" = None,
        knowledge_base: Any | None = None,  # KnowledgeBaseProvider
        knowledge_max_items: int = 3,  # 知识库查询最大条目数
        knowledge_relevance_threshold: float = 0.7,  # 知识相关度阈值
        session_isolation: SessionIsolationMode = SessionIsolationMode.STRICT,  # Session 隔离模式
        compaction_config: "CompactionConfig | None" = None,  # 记忆压缩配置
        segment_store: "SegmentStore | None" = None,  # 片段存储（用于 Fact Indexing）
        tool_policy: "ToolPolicy | None" = None,  # 工具权限策略
        observability_exporters: list[Any] | None = None,  # SpanExporter 列表
        enable_checkpoint: bool = False,  # 启用检查点
        state_store: Any | None = None,  # StateStore（检查点存储后端）
        shared_pool: "SharedMemoryPool | None" = None,  # 跨 Agent 共享记忆池
        embedding_provider: Any | None = None,  # EmbeddingProvider（L3 向量检索）
        **kwargs,
    ):
        """
        初始化智能体

        Args:
            node_id: 节点ID
            llm_provider: LLM提供者
            system_prompt: User's business logic prompt (framework capabilities added automatically)
            tools: 可用工具列表（普通工具）
            available_agents: 可用的其他agent（用于委派）
            event_bus: 事件总线（可选，用于观测和上下文管理）
            enable_observation: 是否启用观测能力
            max_context_tokens: 最大上下文token数
            max_iterations: 最大迭代次数
            require_done_tool: 是否要求显式调用done工具完成任务
            recursive_depth: 当前递归深度（内部使用）
            skill_registry: Skill注册表（可选，用于加载Skills）
            skill_activator: Skill激活器（可选，Phase 2 三种激活形态）
            tool_registry: 工具注册表（可选，用于执行工具调用）
            sandbox_manager: 沙盒工具管理器（可选，用于统一管理工具和自动沙盒化）
            memory_config: 记忆系统配置（可选，默认使用标准配置）
            context_budget_config: 上下文预算配置（可选，动态调整比例）
            knowledge_base: 知识库提供者（可选，用于智能RAG）
            knowledge_max_items: 知识库查询返回的最大条目数（默认3）
            knowledge_relevance_threshold: 知识相关度阈值，0.0-1.0（默认0.7）
            **kwargs: 其他参数传递给BaseNode
        """
        _pending_tool_callables = kwargs.pop("_pending_tool_callables", []) or []
        super().__init__(
            node_id=node_id,
            node_type="agent",
            event_bus=event_bus,
            enable_observation=enable_observation,
            enable_collective_memory=True,
            **kwargs,
        )

        # 规范化 session_isolation（允许传入字符串）
        if isinstance(session_isolation, str):
            try:
                session_isolation = SessionIsolationMode(session_isolation)
            except ValueError as exc:
                raise ValueError(
                    "session_isolation must be one of: strict, advisory, none"
                ) from exc
        elif not isinstance(session_isolation, SessionIsolationMode):
            raise TypeError("session_isolation must be SessionIsolationMode or str")

        # 注入 Session Lane 拦截器（默认 strict 模式）
        self.interceptor_chain.add(SessionLaneInterceptor(mode=session_isolation))

        self.llm_provider = llm_provider

        # system_prompt is built later (after self.memory is created)
        # so that MEMORY_GUIDANCE can be conditionally injected
        self._user_system_prompt = system_prompt

        # Detect if user explicitly wants no tools (tools=[] vs tools=None)
        # This allows tools to be truly optional, not forced
        self._user_wants_no_tools = tools is not None and len(tools) == 0
        self.tools = tools or []
        self.available_agents = available_agents or {}
        self.session = session  # Session 注入（Agent 无状态模式）
        self.max_iterations = max_iterations

        # Auto-disable done_tool requirement when no tools are wanted
        # This prevents infinite loops when LLM has no tools to call
        if self._user_wants_no_tools:
            self.require_done_tool = False
        else:
            self.require_done_tool = require_done_tool

        # === Phase 3: 配置体系（12.5.2）===
        from loom.config.agent import AgentConfig

        # 1. 配置（可继承）
        self.config = config or AgentConfig()

        # 2. 共享的 Registry（全局单例）
        self.skill_registry = skill_registry
        self.tool_registry = tool_registry

        # 3. 激活状态（独立，不继承）
        self._active_skills: set[str] = set()

        # 创建 SkillActivator（如果未提供且有 skill_registry）
        self.skill_activator: "SkillActivator | None"  # noqa: UP037
        if skill_activator is None and skill_registry is not None:
            from loom.tools.skills.activator import SkillActivator

            self.skill_activator = SkillActivator(
                llm_provider=llm_provider,
                tool_registry=tool_registry,
                tool_manager=sandbox_manager,
            )
        else:
            from typing import cast

            self.skill_activator = cast(
                "SkillActivator | None", skill_activator
            )  # Phase 2: Skill激活器

        self.sandbox_manager = sandbox_manager  # NEW: 存储沙盒工具管理器
        self._pending_tool_callables: list[Any] = (
            _pending_tool_callables  # Phase 3 Task 4: 首次执行时注册到 sandbox_manager
        )
        self.knowledge_base = knowledge_base  # 知识库提供者（用于智能RAG，保留原始引用）
        self.knowledge_max_items = knowledge_max_items  # 知识库查询最大条目数
        self.knowledge_relevance_threshold = knowledge_relevance_threshold  # 知识相关度阈值

        # 标准化 knowledge_base 为列表（统一检索工具需要）
        if knowledge_base is None:
            self._knowledge_bases: list[Any] = []
        elif isinstance(knowledge_base, list):
            self._knowledge_bases = knowledge_base
        else:
            self._knowledge_bases = [knowledge_base]
        self._root_context_id = root_context_id
        self._recursive_depth = recursive_depth
        self.shared_pool = shared_pool  # 跨 Agent 共享记忆池

        # 如果启用 done tool，添加到工具列表
        if self.require_done_tool:
            self.tools.append(create_done_tool())

        # 创建 MemoryManager（3 层记忆系统）
        if session is not None:
            # Session 注入模式：Agent 使用 Session 的 LoomMemory
            # memory IS context — 共享 Session 的 L1/L2/L3
            if memory_config is not None:
                logging.warning(
                    "Agent %s: session provided, memory_config ignored", node_id
                )
            self.memory = MemoryManager(
                node_id=node_id,
                parent=parent_memory,
                memory=session.memory,  # 注入 Session 的 LoomMemory
                event_bus=event_bus,
            )
            # 注入 embedding provider 到已有的 LoomMemory
            if embedding_provider:
                self.memory.memory.set_embedding_provider(embedding_provider)
        else:
            # 独立模式：Agent 创建自己的 MemoryManager
            memory_kwargs: dict[str, Any] = {}
            if memory_config is not None:
                if isinstance(memory_config, MemoryConfig):
                    memory_kwargs = {
                        "l1_token_budget": memory_config.l1.capacity,
                        "l2_token_budget": memory_config.l2.capacity,
                    }
                elif isinstance(memory_config, dict):
                    allowed = {
                        "l1_token_budget", "l2_token_budget",
                        "l2_importance_threshold", "l2_ttl_seconds",
                    }
                    if "l1_token_budget" in memory_config or "l2_token_budget" in memory_config:
                        memory_kwargs = {k: v for k, v in memory_config.items() if k in allowed}
                    else:
                        # 提取 allowed 中的 flat keys
                        memory_kwargs = {k: v for k, v in memory_config.items() if k in allowed}
                        if "max_l1_size" in memory_config:
                            memory_kwargs["l1_token_budget"] = memory_config["max_l1_size"]
                        if "max_l2_size" in memory_config:
                            memory_kwargs["l2_token_budget"] = memory_config["max_l2_size"]
                        if "l1" in memory_config:
                            l1 = memory_config["l1"]
                            cap = l1.get("capacity") if isinstance(l1, dict) else getattr(l1, "capacity", None)
                            if cap is not None:
                                memory_kwargs["l1_token_budget"] = cap
                        if "l2" in memory_config:
                            l2 = memory_config["l2"]
                            cap = l2.get("capacity") if isinstance(l2, dict) else getattr(l2, "capacity", None)
                            if cap is not None:
                                memory_kwargs["l2_token_budget"] = cap
                else:
                    raise TypeError("memory_config must be MemoryConfig or dict")

            self.memory = MemoryManager(
                node_id=node_id, parent=parent_memory, event_bus=event_bus,
                embedding_provider=embedding_provider,
                **memory_kwargs,
            )

        # 创建 Observability 组件（Tracing + Metrics）
        self._tracer = LoomTracer(agent_id=node_id, enabled=enable_observation)
        self._metrics = LoomMetrics(agent_id=node_id, enabled=enable_observation)

        # 注册外部导出器（OTEL / Console 等）
        if observability_exporters:
            for exporter in observability_exporters:
                self._tracer.add_exporter(exporter)
        self._metrics_exporters: list[Any] = []
        if observability_exporters:
            from loom.observability.exporters import OTLPMetricsExporter

            self._metrics_exporters = [
                e for e in observability_exporters if isinstance(e, OTLPMetricsExporter)
            ]

        # 创建 CheckpointManager（可选）
        self._checkpoint_mgr = None
        if enable_checkpoint:
            from loom.runtime.checkpoint import CheckpointManager
            from loom.runtime.state_store import MemoryStateStore

            self._checkpoint_mgr = CheckpointManager(state_store or MemoryStateStore())

        # 创建统一检索组件
        from loom.tools.search import UnifiedSearchExecutor, UnifiedSearchToolBuilder

        self._search_executor = UnifiedSearchExecutor(
            memory=self.memory,
            knowledge_bases=self._knowledge_bases or None,
        )
        self._search_tool_builder = UnifiedSearchToolBuilder()

        # 动态工具执行器（始终创建，LLM 自主决定是否使用）
        self._dynamic_tool_executor = DynamicToolExecutor(sandbox_manager=self.sandbox_manager)

        # 创建 ContextOrchestrator（统一的上下文编排器）
        from loom.context import ContextSource
        from loom.context.sources import InheritedSource

        sources: list[ContextSource] = []
        # L2/L3 源通过 ContextOrchestrator 注入；L1 由 ExecutionEngine 直接追加
        _loom_mem = self.memory.memory  # LoomMemory 实例
        sources.append(L2WorkingSource(_loom_mem))
        sources.append(L3PersistentSource(_loom_mem))
        sources.append(InheritedSource(self.memory))

        # 共享记忆池源
        if self.shared_pool is not None:
            from loom.context.sources.shared_pool import SharedPoolSource

            sources.append(SharedPoolSource(self.shared_pool))

        # 统一检索源（替代独立的 RAGKnowledgeSource，整合 L4 语义 + RAG 知识库）
        self._retrieval_source = None
        if self.memory or self._knowledge_bases:
            from loom.context.retrieval import UnifiedRetrievalSource

            _loom_memory = getattr(self.memory, "memory", None) if self.memory else None
            self._retrieval_source = UnifiedRetrievalSource(
                memory=_loom_memory,
                knowledge_bases=self._knowledge_bases or None,
            )
            sources.append(self._retrieval_source)

        # 构建预算分配比例
        allocation_ratios = None
        if context_budget_config is not None and isinstance(context_budget_config, dict):
            allocation_ratios = {
                "L2_working": context_budget_config.get("l2_ratio", 0.20),
                "L3_persistent": context_budget_config.get("l3_ratio", 0.15),
                "INHERITED": context_budget_config.get("inherited_ratio", 0.10),
                "retrieval": context_budget_config.get("retrieval_ratio", 0.15),
            }

        # 使用 AdaptiveBudgetManager（根据任务阶段动态调整预算分配）
        from loom.context.budget import AdaptiveBudgetManager

        _token_counter = TiktokenCounter(model="gpt-4")
        self._adaptive_budget = AdaptiveBudgetManager(
            token_counter=_token_counter,
            model_context_window=max_context_tokens,
            allocation_ratios=allocation_ratios,
        )

        self.context_orchestrator = ContextOrchestrator(
            token_counter=_token_counter,
            sources=sources,
            model_context_window=max_context_tokens,
            budget_manager=self._adaptive_budget,
        )

        # 【Phase 2】创建记忆压缩器
        if compaction_config is None:
            compaction_config = CompactionConfig()
        elif isinstance(compaction_config, dict):
            compaction_config = CompactionConfig(**compaction_config)
        elif not isinstance(compaction_config, CompactionConfig):
            raise TypeError("compaction_config must be CompactionConfig or dict")

        self.memory_compactor = MemoryCompactor(
            config=compaction_config,
            memory_manager=self.memory,
            token_counter=self.context_orchestrator.token_counter,
            segment_store=segment_store,
        )

        # 【Phase 3】存储工具权限策略
        self.tool_policy = tool_policy

        # Build full system prompt (deferred until after self.memory is created
        # so that MEMORY_GUIDANCE can be conditionally injected)
        self.system_prompt = self._build_full_system_prompt(self._user_system_prompt)

        # 构建完整工具列表（普通工具 + 元工具）
        self.all_tools = self._build_tool_list()

        # 提取的执行组件（core.py 瘦身）
        self._tool_router = ToolRouter(self)
        self._engine = ExecutionEngine(self)

        # Ephemeral 消息跟踪（用于大输出工具）
        self._ephemeral_tool_outputs: dict[str, deque] = defaultdict(lambda: deque())

        # EventBus委派处理器已移除（使用统一的 Agent.delegate() 方法）
        # Tier 2 delegation through EventBusDelegationHandler has been removed
        # All delegation now uses Agent.delegate() which internally uses EventBus

        # AgentNode 统一抽象：节点类型
        # BaseNode 已经设置了 self.node_type = "agent"
        # 我们提供 get_node_type() 方法来获取枚举值

    # ==================== 状态检查属性 ====================

    @property
    def active_skills(self) -> set[str]:
        """当前激活的技能集合（只读）"""
        return self._active_skills.copy()

    @property
    def current_task_id(self) -> str | None:
        """当前执行的任务ID（只读）"""
        return getattr(self, "_current_task_id", None)

    @property
    def recursive_depth(self) -> int:
        """当前递归深度（只读）"""
        return self._recursive_depth

    @property
    def root_context_id(self) -> str | None:
        """根上下文ID（只读）"""
        return self._root_context_id

    @property
    def ephemeral_tool_outputs(self) -> dict[str, list[Any]]:
        """临时工具输出历史（只读）"""
        return {k: list(v) for k, v in self._ephemeral_tool_outputs.items()}

    @property
    def dynamic_tools(self) -> list[str]:
        """已创建的动态工具列表（只读）"""
        if self._dynamic_tool_executor:
            return list(self._dynamic_tool_executor.created_tools.keys())
        return []

    def get_agent_state(self) -> dict[str, Any]:
        """
        获取 Agent 完整状态快照（用于调试和可观测性）

        Returns:
            包含所有关键状态的字典
        """
        return {
            "node_id": self.node_id,
            "current_task_id": self.current_task_id,
            "recursive_depth": self.recursive_depth,
            "root_context_id": self.root_context_id,
            "active_skills": list(self.active_skills),
            "dynamic_tools": self.dynamic_tools,
            "ephemeral_outputs_count": {k: len(v) for k, v in self._ephemeral_tool_outputs.items()},
            "pending_tools_count": len(self._pending_tool_callables),
            "max_iterations": self.max_iterations,
            "enabled_skills": list(self.config.enabled_skills) if self.config else [],
        }

    @classmethod
    def create(
        cls,
        llm: LLMProvider,
        *,
        system_prompt: str = "",
        tools: list[dict[str, Any]] | None = None,
        skills: list[str] | None = None,
        capabilities: Any | None = None,
        node_id: str | None = None,
        parent_node_id: str | None = None,  # 父节点ID（用于结构化命名）
        agent_role: str | None = None,  # Agent角色（用于SSE识别）
        event_bus: Any | None = None,
        session: "Session | None" = None,
        knowledge_base: Any | None = None,
        max_context_tokens: int = 4000,
        max_iterations: int = 10,
        tool_policy: ToolPolicy | None = None,
        # v0.5.1: 聚合配置（渐进式披露）
        tool_config: ToolConfig | dict[str, Any] | None = None,
        context_config: ContextConfig | dict[str, Any] | None = None,
        # 以下参数已弃用，请使用 tool_config / context_config
        skills_dir: str | Path | list[str | Path] | None = None,
        skill_loaders: list[Any] | None = None,
        memory_config: MemoryConfig | dict[str, Any] | None = None,
        context_budget_config: Any | None = None,
        session_isolation: SessionIsolationMode | str | None = None,
        compaction_config: CompactionConfig | dict[str, Any] | None = None,
        **kwargs,
    ) -> "Agent":
        """
        创建 Agent 的推荐方式（委派给 AgentFactory）。

        v0.5.1: 支持渐进式披露的聚合配置
        """
        return AgentFactory.create(
            llm,
            system_prompt=system_prompt,
            tools=tools,
            skills=skills,
            capabilities=capabilities,
            node_id=node_id,
            parent_node_id=parent_node_id,
            agent_role=agent_role,
            event_bus=event_bus,
            session=session,
            knowledge_base=knowledge_base,
            max_context_tokens=max_context_tokens,
            max_iterations=max_iterations,
            tool_policy=tool_policy,
            tool_config=tool_config,
            context_config=context_config,
            skills_dir=skills_dir,
            skill_loaders=skill_loaders,
            memory_config=memory_config,
            context_budget_config=context_budget_config,
            session_isolation=session_isolation,
            compaction_config=compaction_config,
            **kwargs,
        )

    @classmethod
    def builder(cls, llm: LLMProvider) -> "AgentBuilder":
        """
        返回 AgentBuilder，支持链式配置后 .build() 得到 Agent。

        Args:
            llm: LLM 提供者

        Returns:
            AgentBuilder 实例

        Examples:
            >>> agent = (Agent.builder(llm)
            ...     .with_system_prompt("你是一个AI助手")
            ...     .with_tools([...])
            ...     .build())
        """
        return AgentBuilder(llm)

    # 兼容旧命名：一步创建
    from_llm = create

    async def run(self, content: str, **kwargs) -> str:
        """
        简化的执行接口 - 直接运行任务并返回结果

        Args:
            content: 任务内容
            **kwargs: 额外参数

        Returns:
            任务执行结果字符串

        Examples:
            >>> result = await agent.run("帮我分析这段代码")
        """
        task = Task(
            taskId=str(uuid4()), action="execute", parameters={"content": content, **kwargs}
        )
        result = await self.execute_task(task)

        if result.status == TaskStatus.COMPLETED:
            if isinstance(result.result, dict):
                return str(result.result.get("content", ""))
            return str(result.result)
        else:
            return f"Error: {result.error or 'Task failed'}"

    def get_node_type(self) -> "NodeType":
        """
        获取节点类型（AgentNode 统一抽象）

        Returns:
            NodeType.AGENT 表示完整四范式执行
        """
        from .agent_node import NodeType

        return NodeType.AGENT

    def _build_full_system_prompt(
        self, user_prompt: str, skill_instructions: list[str] | None = None
    ) -> str:
        """
        Build complete system prompt (user prompt + skills + framework capabilities)

        Architecture:
        - user_prompt: User's business logic and task-specific instructions
        - skill_instructions: Activated Skill instructions (Form 1 - Phase 2)
        - framework_capabilities: Four-paradigm autonomous capabilities (always added, non-configurable)

        Args:
            user_prompt: User's business logic prompt
            skill_instructions: Optional list of Skill instructions to inject (Form 1)

        Returns:
            Complete system prompt
        """
        autonomous_capabilities = """

<autonomous_agent>
You are an autonomous agent using ReAct (Reasoning + Acting) as your PRIMARY working method.

<primary_method>
  <react>
    Your DEFAULT approach for ALL tasks:
    1. Think: Analyze the task
    2. Act: Use available tools directly
    3. Observe: See results
    4. Repeat until completion

    ALWAYS try ReAct first. Most tasks can be solved with direct tool use.
  </react>
</primary_method>

<secondary_methods>
  <planning tool="create_plan">
    ONLY use when task genuinely requires 5+ INDEPENDENT steps.
    <avoid_when>
      - Task can be solved with sequential tool calls (use ReAct instead)
      - Current recursion depth is high (check budget)
      - The step is simple and straightforward
    </avoid_when>
    <when_to_use_in_plan_step>
      - The plan step itself is genuinely complex and requires structured breakdown
      - Breaking down further would significantly improve clarity and execution
      - Sub-planning is justified by complexity, not just convenience
    </when_to_use_in_plan_step>
  </planning>

  <collaboration tool="delegate_task">
    Use when need specialized expertise beyond your tools.
  </collaboration>

  <context_query>
    Query historical information when needed:
    - query (unified search across memory and knowledge bases)
    - query_events (unified event query)
  </context_query>
</secondary_methods>

<decision_framework>
  1. DEFAULT: Use ReAct - directly call tools to solve the task
  2. ONLY if task has 5+ truly independent steps: Consider planning
  3. If executing plan step: Prefer ReAct, but may sub-plan if step is genuinely complex and requires structured breakdown
</decision_framework>

<principles>
- ReAct is your primary method - use tools directly
- Planning is secondary - only for genuinely complex tasks
- Respond in the same language as the user
- Act directly without asking permission
</principles>
</autonomous_agent>
"""

        # Build prompt in layers: user_prompt + skills + framework_capabilities
        parts = []

        if user_prompt:
            parts.append(user_prompt)

        # Inject Skill instructions (Form 1 - Phase 2)
        if skill_instructions:
            skills_section = "\n\n".join(skill_instructions)
            parts.append(skills_section)

        # Inject MEMORY_GUIDANCE when memory or knowledge bases are available
        # Use getattr: this method is called during __init__ before self.memory is assigned
        if getattr(self, "memory", None) or getattr(self, "_knowledge_bases", None):
            parts.append(MEMORY_GUIDANCE.strip())

        # Framework capabilities are always added (non-configurable)
        parts.append(autonomous_capabilities.strip())

        return "\n\n".join(parts)

    async def _build_skill_context(self) -> str:
        """
        构建 Skill 上下文（用于 Discovery 层 - Phase 3: 12.5.4）

        返回所有启用 Skills 的元数据，让 LLM 知道有哪些 Skills 可用。
        这是 Progressive Disclosure 的第二层：Discovery 层。

        Returns:
            Skill 上下文字符串
        """
        if not self.skill_registry or not self.config.enabled_skills:
            return ""

        lines = ["## Available Skills\n"]

        for skill_id in self.config.enabled_skills:
            try:
                # 获取 Skill 定义
                skill_def = await self.skill_registry.get_skill(skill_id)
                if skill_def:
                    # 添加 Skill 基本信息
                    lines.append(f"- **{skill_def.name}**: {skill_def.description}")

                    # 如果有绑定的工具，显示工具列表
                    if skill_def.required_tools:
                        tools_str = ", ".join(skill_def.required_tools)
                        lines.append(f"  - Required Tools: {tools_str}")
            except Exception:
                # 跳过无法加载的 Skill
                continue

        # 如果没有可用的 Skills，返回空字符串
        if len(lines) == 1:
            return ""

        return "\n".join(lines)

    async def _activate_skills(self, task_description: str) -> dict[str, Any]:
        """
        Activate relevant skills for the task.

        Finds and activates skills via INJECTION mode (instructions injected
        into system_prompt).

        Args:
            task_description: Description of the task to find relevant skills

        Returns:
            Dictionary with:
            - injected_instructions: list[str] - Skill instructions to inject
        """
        result: dict[str, Any] = {
            "injected_instructions": [],
        }

        # Check if skill_activator is available
        if not self.skill_activator or not self.skill_registry:
            return result

        try:
            # Step 1: Find relevant skills (Progressive Disclosure)
            # Get skill metadata from registry
            skill_metadata = []
            if hasattr(self.skill_registry, "get_all_metadata"):
                skill_metadata = await self.skill_registry.get_all_metadata()

            # Find relevant skills using LLM
            relevant_skill_ids = await self.skill_activator.find_relevant_skills(
                task_description=task_description,
                skill_metadata=skill_metadata,
                max_skills=5,
            )

            # Step 2: Activate each relevant skill

            for skill_id in relevant_skill_ids:
                skill_def = await self.skill_registry.get_skill(skill_id)
                if not skill_def:
                    continue

                # Activate via injection
                activation_result = await self.skill_activator.activate(skill_def)
                if activation_result.success and activation_result.content:
                    result["injected_instructions"].append(activation_result.content)

        except Exception as e:
            # Log error but don't fail - skills are optional enhancement
            print(f"Error activating skills: {e}")

        return result

    async def _activate_skill(self, skill_id: str) -> dict[str, Any]:
        """
        激活单个 Skill 并验证依赖

        流程：
        1. 检查 skill 是否在 enabled_skills 中
        2. 检查是否已激活（避免重复）
        3. 验证绑定的 tools 是否可用
        4. 激活 Skill（INJECTION 模式）
        5. 记录激活状态

        Args:
            skill_id: Skill ID

        Returns:
            激活结果字典：
            {
                "success": bool,
                "already_active": bool,
                "content": str,  # 注入的指令内容
                "error": str,
            }
        """
        # 1. 检查是否启用
        if skill_id not in self.config.enabled_skills:
            return {
                "success": False,
                "error": f"Skill '{skill_id}' not in enabled_skills",
            }

        # 2. 检查是否已激活
        if skill_id in self._active_skills:
            return {
                "success": True,
                "already_active": True,
            }

        # 3. 使用 SkillActivator 验证并激活
        if not self.skill_activator or not self.skill_registry:
            return {
                "success": False,
                "error": "SkillActivator or SkillRegistry not initialized",
            }

        try:
            # 获取 Skill 定义
            skill_def = await self.skill_registry.get_skill(skill_id)
            if not skill_def:
                return {
                    "success": False,
                    "error": f"Skill '{skill_id}' not found in registry",
                }

            # 激活（带依赖验证）
            from loom.tools.skills.models import ActivationResult

            result: ActivationResult = await self.skill_activator.activate(
                skill=skill_def,
                tool_manager=self.sandbox_manager,
                event_bus=self.event_bus,
            )

            # 4. 记录激活状态
            if result.success:
                self._active_skills.add(skill_id)

            # 5. 转换为字典返回
            return {
                "success": result.success,
                "mode": result.mode,
                "content": result.content,
                "tool_names": result.tool_names,
                "node": result.node,
                "error": result.error,
                "missing_tools": result.missing_tools,
            }

        except Exception as e:
            return {
                "success": False,
                "error": str(e),
            }

    def _get_available_tools(self) -> list[dict[str, Any]]:
        """
        获取当前可用的工具列表（动态 - Phase 3: 12.5.6）

        来源：
        1. 基础工具（self.tools）
        2. 已激活 Skills 绑定的工具（通过 sandbox_manager）
        3. 额外配置的工具（config.extra_tools）

        排除：
        - config.disabled_tools 中的工具

        Returns:
            工具定义列表（LLM 格式）
        """
        available = []
        tool_names_seen = set()

        # 1. 基础工具
        for tool in self.tools:
            if isinstance(tool, dict):
                # OpenAI 格式的工具定义
                tool_name = tool.get("function", {}).get("name")
                if tool_name and tool_name not in tool_names_seen:
                    available.append(tool)
                    tool_names_seen.add(tool_name)
            elif hasattr(tool, "name") and hasattr(tool, "input_schema"):
                # MCPToolDefinition 或类似的对象（支持 FunctionToMCP.convert 返回值）
                tool_name = tool.name
                if tool_name and tool_name not in tool_names_seen:
                    available.append(
                        {
                            "type": "function",
                            "function": {
                                "name": tool.name,
                                "description": getattr(tool, "description", ""),
                                "parameters": tool.input_schema,
                            },
                        }
                    )
                    tool_names_seen.add(tool_name)
            # Skip other types (e.g., raw functions from old tests)

        # 2. 已激活 Skills 绑定的工具
        # 注意：Skills 的工具已经通过 sandbox_manager 注册，会在后面统一添加

        # 3. 额外配置的工具（从 tool_registry）
        if self.tool_registry and self.config.extra_tools:
            for tool_name in self.config.extra_tools:
                if tool_name not in tool_names_seen:
                    tool_def = self.tool_registry.get_definition(tool_name)
                    if tool_def:
                        # 转换为 LLM 格式
                        available.append(
                            {
                                "type": "function",
                                "function": {
                                    "name": tool_def.name,
                                    "description": tool_def.description,
                                    "parameters": tool_def.input_schema,
                                },
                            }
                        )
                        tool_names_seen.add(tool_name)

        # 4. 沙盒工具（sandbox_manager，含已激活 Skills 绑定的工具）
        if self.sandbox_manager:
            for mcp_def in self.sandbox_manager.list_tools():
                if mcp_def.name not in tool_names_seen:
                    tool_names_seen.add(mcp_def.name)
                    available.append(
                        {
                            "type": "function",
                            "function": {
                                "name": mcp_def.name,
                                "description": mcp_def.description,
                                "parameters": mcp_def.input_schema,
                            },
                        }
                    )

        # 过滤禁用的工具
        if self.config.disabled_tools:
            available = [
                tool
                for tool in available
                if isinstance(tool, dict)
                and tool.get("function", {}).get("name") not in self.config.disabled_tools
            ]

        return available

    def _build_tool_list(self) -> list[dict[str, Any]]:
        """
        构建完整工具列表（普通工具 + 元工具）

        如果用户明确传入 tools=[]，则不添加任何元工具，
        让工具成为真正可选的增强能力。

        Returns:
            完整的工具列表
        """
        # 使用动态工具列表（基于配置和已激活的 Skills）
        tools = self._get_available_tools()

        # If user explicitly wants no tools (tools=[]), respect that choice
        # Tools should be optional enhancements, not forced dependencies
        if self._user_wants_no_tools:
            return tools

        # 获取禁用的工具列表
        disabled = self.config.disabled_tools if self.config else set()

        # 添加规划元工具
        if "create_plan" not in disabled:
            tools.append(
                {
                    "type": "function",
                    "function": {
                        "name": "create_plan",
                        "description": (
                            "Create execution plan for complex tasks. "
                            "Use when: task requires 3+ independent steps, multi-stage workflows, cross-domain tasks. "
                            "Avoid when: single-step tasks, already executing plan step (avoid nesting), simple operations, deep recursion. "
                            "Final decision is yours based on actual complexity."
                        ),
                        "parameters": {
                            "type": "object",
                            "properties": {
                                "goal": {"type": "string", "description": "Goal to achieve"},
                                "steps": {
                                    "type": "array",
                                    "items": {"type": "string"},
                                    "description": "List of execution steps",
                                },
                                "reasoning": {
                                    "type": "string",
                                    "description": "Why this plan is needed",
                                },
                            },
                            "required": ["goal", "steps"],
                        },
                    },
                }
            )

        # 添加分形委派元工具（自动创建子节点）
        if "delegate_task" not in disabled:
            from loom.agent.delegator import create_delegate_task_tool

            tools.append(create_delegate_task_tool())

        # 添加委派元工具（如果有可用的agents）
        if self.available_agents and "delegate_to_agent" not in disabled:
            agent_list = ", ".join(self.available_agents.keys())
            tools.append(
                {
                    "type": "function",
                    "function": {
                        "name": "delegate_to_agent",
                        "description": f"将子任务委派给指定的专业agent。可用的agents: {agent_list}",
                        "parameters": {
                            "type": "object",
                            "properties": {
                                "target_agent": {
                                    "type": "string",
                                    "description": "目标agent的ID",
                                    "enum": list(self.available_agents.keys()),
                                },
                                "subtask": {"type": "string", "description": "要委派的子任务描述"},
                                "reasoning": {
                                    "type": "string",
                                    "description": "为什么需要委派这个任务",
                                },
                            },
                            "required": ["target_agent", "subtask"],
                        },
                    },
                }
            )

        # 添加统一检索工具（支持记忆 + 知识库）
        if "query" not in disabled and (self.memory or self._knowledge_bases):
            query_tool = self._search_tool_builder.build_tool(
                knowledge_bases=self._knowledge_bases or None,
                memory_enabled=self.memory is not None,
            )
            tools.append(query_tool)

        # 添加工具创建元工具（始终提供，LLM 自主决定是否使用）
        if "create_tool" not in disabled:
            tools.append(create_tool_creation_tool())
        tools.extend(self._dynamic_tool_executor.get_tool_definitions())

        return tools

    async def _execute_single_tool(self, tool_name: str, tool_args: dict | str) -> str:
        """
        执行单个工具（委派给 ToolRouter）

        Args:
            tool_name: 工具名称
            tool_args: 工具参数（可能是dict或JSON字符串）

        Returns:
            工具执行结果
        """
        return await self._tool_router.route(tool_name, tool_args)

    async def _ensure_pending_tools_registered(self) -> None:
        """
        Phase 3 Task 4: 将 Agent.create(tools=[callable, ...]) 传入的可调用对象
        在首次执行时注册到 sandbox_manager，保证「发给 LLM 的工具列表」与「执行来源」一致。
        """
        if not self._pending_tool_callables or self.sandbox_manager is None:
            return
        import asyncio

        from loom.tools.core.converters import FunctionToMCP
        from loom.tools.core.sandbox_manager import ToolScope

        pending = list(self._pending_tool_callables)
        self._pending_tool_callables = []
        for func in pending:
            name = getattr(func, "__name__", "unknown")
            definition = FunctionToMCP.convert(func, name=name)
            if not asyncio.iscoroutinefunction(func):
                # Use factory to avoid closure capture issues
                def _create_wrapper(f):
                    async def _wrapped(*args: Any, **kwargs: Any) -> Any:
                        return f(*args, **kwargs)

                    return _wrapped

                exec_func: Any = _create_wrapper(func)
            else:
                exec_func = func
            await self.sandbox_manager.register_tool(
                name=name,
                func=exec_func,
                definition=definition,
                scope=ToolScope.SYSTEM,
            )
        self.all_tools = self._build_tool_list()

    async def _execute_impl(self, task: Task) -> Task:
        """
        执行任务 - 委派给 ExecutionEngine

        核心理念：Agent is just a for loop

        Args:
            task: 任务

        Returns:
            更新后的任务
        """
        return await self._engine.run(task)

    # ==================== Ephemeral 消息过滤 ====================

    def _get_tool_ephemeral_count(self, tool_name: str) -> int:
        """
        获取工具的 ephemeral 设置

        Args:
            tool_name: 工具名称

        Returns:
            ephemeral 计数（0 表示不是 ephemeral 工具）
        """
        for tool in self.all_tools:
            if isinstance(tool, dict) and tool.get("function", {}).get("name") == tool_name:
                ephemeral = tool.get("_ephemeral", 0)
                return int(ephemeral) if isinstance(ephemeral, int | float) else 0
        return 0

    def _filter_ephemeral_messages(
        self,
        messages: list[dict[str, str]],
    ) -> list[dict[str, str]]:
        """
        过滤 ephemeral 消息，只保留最近的

        策略：
        1. 识别每个 ephemeral 工具的输出消息
        2. 只保留最近 N 次输出
        3. 丢弃旧的输出

        Args:
            messages: 消息列表

        Returns:
            过滤后的消息列表
        """
        if not messages:
            return messages

        # 统计每个 ephemeral 工具的出现次数
        tool_counts: dict[str, int] = defaultdict(int)
        filtered = []

        # 反向遍历（从最新到最旧）
        for msg in reversed(messages):
            tool_name = msg.get("tool_name")

            if tool_name:
                # 这是工具输出消息
                ephemeral_count = self._get_tool_ephemeral_count(tool_name)

                if ephemeral_count > 0:
                    # 这是 ephemeral 工具
                    tool_counts[tool_name] += 1

                    if tool_counts[tool_name] <= ephemeral_count:
                        # 在保留范围内
                        filtered.append(msg)
                    # else: 丢弃这条消息
                else:
                    # 普通工具，保留
                    filtered.append(msg)
            else:
                # 非工具消息，保留
                filtered.append(msg)

        # 恢复正序
        filtered.reverse()
        return filtered

    # ==================== 自我评估 ====================

    async def _self_evaluate(self, task: Task) -> None:
        """
        自我评估任务执行结果

        Agent完成任务后，用自己的LLM评估结果质量，
        将质量指标附加到task.result中。

        Args:
            task: 已完成的任务
        """
        if not isinstance(task.result, dict):
            return

        task_content = task.parameters.get("content", "")
        result_content = task.result.get("content", "")

        if not task_content or not result_content:
            return

        prompt = f"""请评估以下任务执行结果的质量。

任务：{task_content}

结果：{result_content[:1000]}

请从三个维度评估（0-1分）：
1. confidence: 结果是否准确完整
2. coverage: 是否完整回答任务要求
3. novelty: 提供了多少有价值的信息

返回JSON：{{"confidence": 0.X, "coverage": 0.X, "novelty": 0.X}}"""

        try:
            response = await self.llm_provider.chat(
                messages=[{"role": "user", "content": prompt}],
                max_tokens=200,
            )
            # 解析响应
            import re

            json_match = re.search(r"\{[^}]+\}", response.content)
            if json_match:
                metrics = json.loads(json_match.group())
                task.result["quality_metrics"] = {
                    "confidence": float(metrics.get("confidence", 0.5)),
                    "coverage": float(metrics.get("coverage", 0.5)),
                    "novelty": float(metrics.get("novelty", 0.5)),
                }
        except Exception:
            # 评估失败不影响任务结果
            pass

    # ==================== 自动能力（内部方法）====================

    async def _load_relevant_skills(self, task_description: str) -> dict[str, Any]:
        """
        加载并激活与任务相关的Skills（Phase 2 - 三种形态）

        使用Progressive Disclosure + LLM智能判断：
        1. 第一阶段：获取所有Skills的元数据（name + description）
        2. 使用LLM判断哪些Skills相关
        3. 第三阶段：激活相关Skills（三种形态）

        Args:
            task_description: 任务描述

        Returns:
            激活结果字典：
            {
                "injected_instructions": list[str],  # Form 1
                "compiled_tools": list[str],         # Form 2
                "instantiated_nodes": list[Any],     # Form 3
            }
        """
        # 如果没有 skill_activator，返回空结果
        if not self.skill_activator:
            return {
                "injected_instructions": [],
                "compiled_tools": [],
                "instantiated_nodes": [],
            }

        # 调用 Phase 2 实现的 _activate_skills() 方法
        # 它会处理所有三种形态的激活
        return await self._activate_skills(task_description)

    async def _execute_delegate_task(
        self,
        target_agent_id: str,
        subtask: str,
        parent_task_id: str,
        session_id: str | None = None,
    ) -> str:
        """
        执行委派任务 - 最小连接机制

        两层机制：
        1. Tier 1（默认）：直接引用 - 通过 available_agents 直接调用
        2. Tier 2（可选）：EventBus 路由 - 通过事件总线解耦

        Args:
            target_agent_id: 目标 agent ID
            subtask: 子任务描述
            parent_task_id: 父任务 ID

        Returns:
            委派结果字符串
        """
        # Tier 1: 直接引用（默认机制）
        if target_agent_id in self.available_agents:
            target_agent = self.available_agents[target_agent_id]

            # 创建委派任务
            delegated_task = Task(
                taskId=f"{parent_task_id}:delegated:{target_agent_id}",
                sourceAgent=self.node_id,
                targetAgent=target_agent_id,
                action="execute",
                parameters={"content": subtask},
                parentTaskId=parent_task_id,
                sessionId=session_id,
            )

            # 直接调用目标 agent
            try:
                result_task = await target_agent.execute_task(delegated_task)

                if result_task.status == TaskStatus.COMPLETED:
                    # 提取结果内容
                    if isinstance(result_task.result, dict):
                        content = result_task.result.get("content", str(result_task.result))
                        return str(content)
                    else:
                        return str(result_task.result)
                else:
                    return f"Delegation failed: {result_task.error or 'Unknown error'}"

            except Exception as e:
                return f"Delegation error: {str(e)}"

        # 找不到目标 agent（既不在 available_agents 中）
        return f"Error: Agent '{target_agent_id}' not found in available_agents"

    async def _execute_plan(
        self,
        plan_args: dict[str, Any],
        parent_task: Task,
    ) -> str:
        """
        执行规划 - 实现Planning范式

        将复杂任务分解为多个子任务，使用分形架构并行/顺序执行

        Args:
            plan_args: 规划参数 {goal, steps, reasoning}
            parent_task: 父任务

        Returns:
            执行结果摘要
        """
        from uuid import uuid4

        goal = plan_args.get("goal", "")
        steps = plan_args.get("steps", [])
        reasoning = plan_args.get("reasoning", "")

        if not steps:
            return "Error: No steps provided in plan"

        # 发布规划事件
        await self._publish_event(
            action="node.planning",
            parameters={
                "goal": goal,
                "steps": steps,
                "reasoning": reasoning,
                "step_count": len(steps),
            },
            task_id=parent_task.taskId,
            session_id=parent_task.sessionId,
        )

        # 将计划写入记忆
        plan_content = f"[Parent Plan] Goal: {goal}\n"
        if reasoning:
            plan_content += f"Reasoning: {reasoning}\n"
        plan_content += f"Steps ({len(steps)}):\n"
        for idx, step in enumerate(steps, 1):
            plan_content += f"  {idx}. {step}\n"

        plan_entry_id = f"plan:{parent_task.taskId}"
        await self.memory.add_context(plan_entry_id, plan_content)

        # DEBUG: 验证写入成功
        import logging

        logger = logging.getLogger(__name__)
        logger.info(f"[DEBUG] Plan written: {plan_entry_id}")

        # 确保父任务上下文可继承
        parent_context_id = await self._ensure_shared_task_context(parent_task)
        root_context_id = parent_task.parameters.get("root_context_id") or self._root_context_id
        context_hints = [cid for cid in [root_context_id, parent_context_id] if cid]

        # 执行每个步骤（分形执行）
        results = []

        # 构建父计划摘要（用于注入子节点）
        parent_plan_summary = f"Goal: {goal}\n"
        if reasoning:
            parent_plan_summary += f"Reasoning: {reasoning}\n"
        parent_plan_summary += f"Steps ({len(steps)}):\n"
        for idx_summary, step_summary in enumerate(steps, 1):
            parent_plan_summary += f"  {idx_summary}. {step_summary}\n"

        for idx, step in enumerate(steps):
            # 创建子任务（标记为计划步骤，并传递父计划）
            subtask = Task(
                taskId=f"{parent_task.taskId}-step-{idx + 1}-{uuid4()}",
                action="execute",
                parameters={
                    "content": step,
                    "parent_task_id": parent_task.taskId,
                    "step_index": idx + 1,
                    "total_steps": len(steps),
                    "is_plan_step": True,  # 标记这是一个计划步骤
                    "parent_plan": parent_plan_summary,  # 传递父计划
                    "root_context_id": root_context_id,
                },
                sessionId=parent_task.sessionId,
            )

            # 创建子节点并执行
            child_node = await self._create_child_node(
                context_hints=context_hints,
            )

            result = await child_node.execute_task(subtask)

            # 同步记忆
            await self._sync_memory_from_child(child_node)

            # 收集结果
            if result.status == TaskStatus.COMPLETED:
                step_result = (
                    result.result.get("content", str(result.result))
                    if isinstance(result.result, dict)
                    else str(result.result)
                )
                results.append(f"Step {idx + 1}: {step_result}")
            else:
                results.append(f"Step {idx + 1}: Failed - {result.error or 'Unknown error'}")

        # 聚合结果 - 使用LLM综合生成最终答案
        # 构建步骤执行结果的上下文
        steps_context = "\n".join(results)

        # 获取用户的原始问题
        original_question = parent_task.parameters.get("content", goal)

        # 调用LLM综合生成最终答案
        # 简化语言处理：在提示词中要求使用用户的语言回答
        synthesis_prompt = f"""You have executed a plan to answer the user's question. Now provide a direct, comprehensive answer based on the execution results.

User's Original Question: {original_question}

Plan Execution Results:
{steps_context}

IMPORTANT:
- Provide a DIRECT answer to the user's question
- Do NOT describe the plan or say "I created a plan"
- Synthesize insights from the execution results
- Focus on actionable recommendations and analysis
- Write as if you're directly answering the question, not summarizing a process
- MUST respond in the same language as the user's original question"""

        try:
            synthesis_response = await self.llm_provider.chat(
                messages=[{"role": "user", "content": synthesis_prompt}],
                max_tokens=1000,
            )
            final_answer = synthesis_response.content
        except Exception:
            # 如果综合失败，返回步骤摘要作为后备
            final_answer = f"Plan '{goal}' completed with {len(steps)} steps:\n" + steps_context

        # 直接抛出 TaskComplete，让 synthesis 结果成为最终答案
        # 这样避免 LLM 再次处理并生成错误的总结
        raise TaskComplete(message=final_answer)

    async def delegate(
        self,
        subtask: str,
        target_node_id: str | None = None,
        **kwargs,
    ) -> Task:
        """
        委派任务给其他节点（AgentNode 统一接口）

        遵循 A2 公理：通过 EventBus 发布任务，不直接调用子节点。

        Args:
            subtask: 子任务描述
            target_node_id: 目标节点 ID（None 表示自动选择）
            **kwargs: 额外参数

        Returns:
            委派结果任务

        Note:
            这是 AgentNode 统一接口的方法。
            与 _auto_delegate 不同，此方法通过 EventBus 进行委派。
        """
        from uuid import uuid4

        from loom.runtime import Task

        # 如果没有指定目标，使用现有的 _auto_delegate 逻辑
        if target_node_id is None:
            # 使用现有的直接委派机制（向后兼容）
            # 获取当前任务（从执行上下文）
            # 这里我们需要模拟一个 parent_task，但由于此方法可能被直接调用，
            # 我们创建一个临时任务对象
            parent_task = Task(
                taskId=f"{self.node_id}-delegate-{uuid4()}",
                action="execute",
                parameters={
                    "content": subtask,
                    **kwargs,
                },
                sourceAgent="user",
                targetAgent=self.node_id,
            )

            # 调用现有的 _auto_delegate 方法
            result_str = await self._auto_delegate(
                {"subtask_description": subtask, **kwargs},
                parent_task,
            )

            # 将结果转换为 Task 对象
            result_task = Task(
                taskId=parent_task.taskId + ":result",
                action="execute",
                parameters={"result": result_str},
                status=TaskStatus.COMPLETED,
                result={"content": result_str},
            )

            return result_task

        # 如果指定了目标，通过 EventBus 发布任务
        if not self.event_bus:
            raise RuntimeError("Cannot delegate: no event_bus available")

        # 创建委派任务
        delegation_task = Task(
            taskId=str(uuid4()),
            sourceAgent=self.node_id,
            targetAgent=target_node_id,
            action="execute",
            parameters={"task": subtask, **kwargs},
            parentTaskId=getattr(self, "_current_task_id", None),
            sessionId=getattr(self, "_session_id", None),
        )

        # 通过 EventBus 发布（遵循 A2 公理）
        from typing import cast

        result = await self.event_bus.publish(delegation_task, wait_result=True)

        return cast(Task, result)

    async def _auto_delegate(
        self,
        args: dict[str, Any],
        parent_task: Task,
    ) -> str:
        """
        自动委派实现（框架内部）

        整合点：
        - 使用 MemoryManager 父子关系（parent_memory）
        - 子节点通过 parent_memory 自动继承父节点的 SHARED/GLOBAL 记忆

        Args:
            args: delegate_task工具参数
            parent_task: 父任务

        Returns:
            子任务执行结果
        """
        from uuid import uuid4

        # 验证必需参数（支持两种参数名）
        subtask_description = args.get("subtask_description") or args.get("subtask")
        if not subtask_description:
            return "Error: subtask_description (or subtask) is required for delegation. Please provide a clear description of the subtask."

        # 1. 创建子任务
        root_context_id = parent_task.parameters.get("root_context_id") or self._root_context_id
        subtask = Task(
            taskId=f"{parent_task.taskId}-child-{uuid4()}",
            action="execute",
            parameters={
                "content": subtask_description,
                "parent_task_id": parent_task.taskId,
                "root_context_id": root_context_id,
            },
            sessionId=parent_task.sessionId,
        )

        # 2. 创建子节点（使用_create_child_node）
        parent_context_id = await self._ensure_shared_task_context(parent_task)
        context_hints = list(args.get("context_hints", []) or [])
        for cid in (root_context_id, parent_context_id):
            if cid and cid not in context_hints:
                context_hints.append(cid)

        child_node = await self._create_child_node(
            context_hints=context_hints,
        )

        # 3. 执行子任务
        result = await child_node.execute_task(subtask)

        # 4. 同步记忆（双向流动）
        await self._sync_memory_from_child(child_node)

        # 5. 返回结果
        if result.status == TaskStatus.COMPLETED:
            if isinstance(result.result, dict):
                return str(result.result.get("content", str(result.result)))
            else:
                return str(result.result)
        else:
            return f"Delegation failed: {result.error or 'Unknown error'}"

    async def _create_child_node(
        self,
        context_hints: list[str] | None = None,
        **kwargs: Any,
    ) -> "Agent":
        """
        创建子节点

        整合所有组件：
        - MemoryManager（继承父节点 parent_memory，自动继承 SHARED/GLOBAL 记忆）
        - AgentConfig（配置继承 - Phase 3）

        继承规则（12.5.3）：
        - 共享：skill_registry, tool_registry, event_bus, sandbox_manager
        - 继承：config (可覆盖)
        - 独立：active_skills, memory（每个节点独立的 MemoryManager）

        Args:
            subtask: 子任务
            context_hints: 上下文提示（记忆ID列表，当前未使用）
            add_skills: 子节点额外启用的 Skills
            remove_skills: 子节点禁用的 Skills
            add_tools: 子节点额外启用的工具
            remove_tools: 子节点禁用的工具

        Returns:
            配置好的子Agent实例
        """
        # 从 kwargs 提取参数
        add_skills: set[str] | None = kwargs.get("add_skills")
        remove_skills: set[str] | None = kwargs.get("remove_skills")
        add_tools: set[str] | None = kwargs.get("add_tools")
        remove_tools: set[str] | None = kwargs.get("remove_tools")
        subtask: Task | None = kwargs.get("subtask")

        # === Phase 3: 配置继承（12.5.3）===
        from loom.config.agent import AgentConfig

        child_config = AgentConfig.inherit(
            parent=self.config,
            add_skills=add_skills,
            remove_skills=remove_skills,
            add_tools=add_tools,
            remove_tools=remove_tools,
        )

        # 1. 为子节点提供上下文（不限制能力，只提供信息）
        child_system_prompt = self.system_prompt
        if subtask and subtask.parameters.get("is_plan_step"):
            parent_plan = subtask.parameters.get("parent_plan", "")
            step_index = subtask.parameters.get("step_index", "?")
            total_steps = subtask.parameters.get("total_steps", "?")

            # 简化的上下文提示：提供信息，不限制能力
            step_context = f"""

<context>
You are executing step {step_index}/{total_steps} of a parent plan.
Parent plan: {parent_plan}
Your task: {subtask.parameters.get("content", "")}

You have full capabilities including planning if genuinely needed.
Prefer ReAct (direct tool use) for most tasks.
</context>
"""
            child_system_prompt = self.system_prompt + step_context

        # 2. 创建子 Agent（parent_memory=self.memory，子节点拥有独立 MemoryManager）
        # 子节点的 MemoryManager 会自动通过 parent_memory 继承父节点的 SHARED/GLOBAL 记忆
        child_node_id = subtask.taskId if subtask else f"{self.node_id}-child-{uuid4()}"
        child_agent = Agent(
            node_id=child_node_id,
            llm_provider=self.llm_provider,
            system_prompt=child_system_prompt,
            tools=self.tools,
            event_bus=self.event_bus,
            max_iterations=self.max_iterations,
            require_done_tool=self.require_done_tool,
            recursive_depth=self._recursive_depth + 1,
            parent_memory=self.memory,
            root_context_id=self._root_context_id,
            skill_registry=self.skill_registry,
            tool_registry=self.tool_registry,
            sandbox_manager=self.sandbox_manager,
            skill_activator=self.skill_activator,
            config=child_config,
            shared_pool=self.shared_pool,
        )

        return child_agent

    async def _ensure_shared_task_context(self, task: Task) -> str | None:
        """
        将当前任务内容写入 SHARED 作用域，供子节点继承
        """
        content = task.parameters.get("content", "")
        if not content:
            return None

        entry_id = f"task:{task.taskId}:content"

        if self._recursive_depth == 0 and task.sessionId:
            root_entry_id = f"session:{task.sessionId}:goal"
            await self.memory.add_context(root_entry_id, content)
            self._root_context_id = root_entry_id
            if "root_context_id" not in task.parameters:
                task.parameters["root_context_id"] = root_entry_id

        existing = await self.memory.read(entry_id)
        if not existing:
            await self.memory.add_context(entry_id, content)
        return entry_id

    async def _sync_memory_from_child(self, child_agent: "Agent") -> None:
        """
        从子节点同步记忆（新架构：通过 parent_memory 自动继承）
        """
        # 新架构：通过 parent_memory 关系自动继承，无需手动同步
        pass



"""
ExecutorMixin - 执行循环

处理任务执行、LLM交互、消息过滤等功能。

从 core.py 拆分，遵循单一职责原则。
"""

from typing import TYPE_CHECKING, Any

from loom.runtime import Task

if TYPE_CHECKING:
    pass


class ExecutorMixin:
    """
    执行处理混入类

    提供执行相关的所有功能：
    - 任务执行循环
    - LLM 交互
    - 消息过滤
    - 自我评估
    """

    # 类型提示（由 Agent 类提供）
    node_id: str
    llm_provider: Any
    memory: Any
    event_bus: Any
    all_tools: list[dict[str, Any]]
    max_iterations: int
    require_done_tool: bool
    _ephemeral_tool_config: dict[str, int]

    def _get_tool_ephemeral_count(self, tool_name: str) -> int:
        """获取工具的 ephemeral 消息保留数量"""
        return self._ephemeral_tool_config.get(tool_name, 0)

    def _filter_ephemeral_messages(
        self,
        messages: list[dict[str, Any]],
    ) -> list[dict[str, Any]]:
        """
        过滤 ephemeral 工具消息

        对于配置了 ephemeral 的工具，只保留最近 N 次调用的消息。
        """
        if not self._ephemeral_tool_config:
            return messages

        # 收集需要过滤的工具调用
        tool_messages: dict[str, list[int]] = {}
        for idx, msg in enumerate(messages):
            if msg.get("role") == "assistant" and msg.get("tool_calls"):
                for tc in msg["tool_calls"]:
                    tool_name = tc.get("function", {}).get("name", "")
                    if tool_name in self._ephemeral_tool_config:
                        if tool_name not in tool_messages:
                            tool_messages[tool_name] = []
                        tool_messages[tool_name].append(idx)

        # 确定要移除的消息索引
        indices_to_remove: set[int] = set()
        for tool_name, indices in tool_messages.items():
            keep_count = self._ephemeral_tool_config[tool_name]
            if len(indices) > keep_count:
                for idx in indices[:-keep_count]:
                    indices_to_remove.add(idx)
                    # 同时移除对应的 tool 响应
                    if idx + 1 < len(messages) and messages[idx + 1].get("role") == "tool":
                        indices_to_remove.add(idx + 1)

        return [msg for idx, msg in enumerate(messages) if idx not in indices_to_remove]

    async def _self_evaluate(self, task: Task) -> None:
        """
        自我评估 - 检查任务是否真正完成

        在 require_done_tool=False 时，LLM 可能过早结束。
        此方法让 LLM 自我检查是否真正完成了任务。
        """
        evaluation_prompt = f"""Review your response to the user's request.

Original request: {task.parameters.get("content", "")}

Did you fully address the request? If not, what's missing?
If complete, respond with "TASK_COMPLETE".
If incomplete, explain what still needs to be done."""

        try:
            response = await self.llm_provider.chat(
                messages=[{"role": "user", "content": evaluation_prompt}],
                max_tokens=500,
            )

            if "TASK_COMPLETE" not in response.content:
                # 任务未完成，继续执行
                pass
        except Exception:
            pass



"""
MCP Provider Interface - MCP 提供者接口

定义 MCP 提供者的统一接口，用于连接和管理 MCP 服务器。

MCP Provider 负责：
1. 连接管理 - 建立和维护与 MCP 服务器的连接
2. 工具发现 - 发现和加载 MCP 服务器提供的工具
3. 工具调用 - 执行 MCP 工具
4. 资源访问 - 访问 MCP 资源
5. 生命周期管理 - 连接、断开、重连
"""

from abc import ABC, abstractmethod
from typing import Any

from loom.tools.mcp_types import MCPPrompt, MCPResource, MCPToolDefinition, MCPToolResult


class MCPProvider(ABC):
    """
    MCP 提供者抽象接口

    所有 MCP 客户端实现都应该继承此接口。
    """

    def __init__(self, provider_id: str):
        """
        初始化 MCP 提供者

        Args:
            provider_id: 提供者唯一标识
        """
        self.provider_id = provider_id
        self._connected = False

    @abstractmethod
    async def connect(self) -> None:
        """
        连接到 MCP 服务器

        Raises:
            ConnectionError: 如果连接失败
        """
        pass

    @abstractmethod
    async def disconnect(self) -> None:
        """
        断开与 MCP 服务器的连接
        """
        pass

    @abstractmethod
    async def list_tools(self) -> list[MCPToolDefinition]:
        """
        列出所有可用工具

        Returns:
            工具定义列表

        Raises:
            RuntimeError: 如果未连接
        """
        pass

    @abstractmethod
    async def call_tool(self, name: str, arguments: dict[str, Any]) -> MCPToolResult:
        """
        调用指定工具

        Args:
            name: 工具名称
            arguments: 工具参数

        Returns:
            工具执行结果

        Raises:
            RuntimeError: 如果未连接
            ValueError: 如果工具不存在
        """
        pass

    @abstractmethod
    async def list_resources(self) -> list[MCPResource]:
        """
        列出所有可用资源

        Returns:
            资源定义列表

        Raises:
            RuntimeError: 如果未连接
        """
        pass

    @abstractmethod
    async def read_resource(self, uri: str) -> str:
        """
        读取资源内容

        Args:
            uri: 资源 URI

        Returns:
            资源内容

        Raises:
            RuntimeError: 如果未连接
            ValueError: 如果资源不存在
        """
        pass

    @abstractmethod
    async def list_prompts(self) -> list[MCPPrompt]:
        """
        列出所有可用提示模板

        Returns:
            提示模板列表

        Raises:
            RuntimeError: 如果未连接
        """
        pass

    @abstractmethod
    async def get_prompt(self, name: str, arguments: dict[str, Any]) -> str:
        """
        获取提示模板内容

        Args:
            name: 模板名称
            arguments: 模板参数

        Returns:
            渲染后的提示内容

        Raises:
            RuntimeError: 如果未连接
            ValueError: 如果模板不存在
        """
        pass

    @property
    def is_connected(self) -> bool:
        """检查是否已连接"""
        return self._connected

    async def __aenter__(self):
        """异步上下文管理器入口"""
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器退出"""
        await self.disconnect()



"""
Memory Reranker - 两阶段检索重排序

解决 L4 语义检索中向量相似度 ≠ 任务相关性的问题。

两阶段策略：
1. 向量召回（Recall）：快速从向量库中召回 top-N 候选
2. 精排（Rerank）：基于多维信号对候选进行重排序

多维信号：
- 向量相似度（基础分）
- 时间衰减（近期记忆优先）
- 访问频率（高频使用的记忆更重要）
- 信息密度（token 效率）
- 任务相关性（与当前任务的语义匹配）
"""

from __future__ import annotations

import math
import time
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Any

from loom.memory.types import WorkingMemoryEntry
from loom.memory.vector_store import VectorSearchResult

# MemoryUnit 类型已移除，使用 WorkingMemoryEntry 代替
MemoryUnit = WorkingMemoryEntry  # type: ignore[misc]


@dataclass
class RerankCandidate:
    """重排序候选项"""

    memory: MemoryUnit
    vector_score: float = 0.0
    rerank_score: float = 0.0
    signal_scores: dict[str, float] = field(default_factory=dict)


@dataclass
class RerankResult:
    """重排序结果"""

    candidates: list[RerankCandidate]
    recall_count: int = 0
    rerank_count: int = 0
    elapsed_ms: float = 0.0

    @property
    def top(self) -> MemoryUnit | None:
        return self.candidates[0].memory if self.candidates else None

    @property
    def memories(self) -> list[MemoryUnit]:
        return [c.memory for c in self.candidates]


# ============ 重排序信号 ============


class RerankSignal(ABC):
    """重排序信号基类"""

    @property
    @abstractmethod
    def name(self) -> str: ...

    @property
    @abstractmethod
    def weight(self) -> float: ...

    @abstractmethod
    def score(self, candidate: RerankCandidate, context: dict[str, Any]) -> float:
        """计算信号分数 (0.0 - 1.0)"""
        ...


class VectorSimilaritySignal(RerankSignal):
    """向量相似度信号（直接使用召回分数）"""

    @property
    def name(self) -> str:
        return "vector_similarity"

    @property
    def weight(self) -> float:
        return 0.30

    def score(self, candidate: RerankCandidate, _context: dict[str, Any]) -> float:
        return max(0.0, min(1.0, candidate.vector_score))


class TimeDecaySignal(RerankSignal):
    """
    时间衰减信号

    使用指数衰减：score = exp(-λ * hours_ago)
    半衰期默认 48 小时（2天前的记忆分数降为 0.5）
    """

    def __init__(self, half_life_hours: float = 48.0):
        self._lambda = math.log(2) / half_life_hours

    @property
    def name(self) -> str:
        return "time_decay"

    @property
    def weight(self) -> float:
        return 0.20

    def score(self, candidate: RerankCandidate, _context: dict[str, Any]) -> float:
        created = candidate.memory.created_at
        hours_ago = (time.time() - created.timestamp()) / 3600.0
        if hours_ago < 0:
            hours_ago = 0
        return math.exp(-self._lambda * hours_ago)


class AccessFrequencySignal(RerankSignal):
    """
    访问频率信号

    高频访问的记忆更可能是重要的。
    使用对数缩放避免极端值主导。
    """

    @property
    def name(self) -> str:
        return "access_frequency"

    @property
    def weight(self) -> float:
        return 0.15

    def score(self, candidate: RerankCandidate, _context: dict[str, Any]) -> float:
        access_count = candidate.memory.metadata.get("access_count", 0)
        if access_count <= 0:
            return 0.1
        # log1p 缩放：1次→0.69, 10次→0.96, 100次→1.0
        return min(1.0, math.log1p(access_count) / math.log1p(100))


class InformationDensitySignal(RerankSignal):
    """信息密度信号（token 效率）"""

    @property
    def name(self) -> str:
        return "information_density"

    @property
    def weight(self) -> float:
        return 0.15

    def score(self, candidate: RerankCandidate, _context: dict[str, Any]) -> float:
        # information_density 属性可能不存在，使用默认值
        density = getattr(candidate.memory, "information_density", 0.5)
        result: float = max(0.0, min(1.0, density))
        return result


class ImportanceSignal(RerankSignal):
    """记忆重要性信号（直接使用 MemoryUnit.importance）"""

    @property
    def name(self) -> str:
        return "importance"

    @property
    def weight(self) -> float:
        return 0.20

    def score(self, candidate: RerankCandidate, _context: dict[str, Any]) -> float:
        result: float = max(0.0, min(1.0, candidate.memory.importance))
        return result


# ============ 主重排序器 ============


DEFAULT_SIGNALS: list[RerankSignal] = [
    VectorSimilaritySignal(),
    TimeDecaySignal(),
    AccessFrequencySignal(),
    InformationDensitySignal(),
    ImportanceSignal(),
]


class MemoryReranker:
    """
    记忆重排序器

    两阶段检索：
    1. 向量召回 top-N 候选（由外部调用方完成）
    2. 多信号加权重排序

    用法：
        reranker = MemoryReranker()
        result = reranker.rerank(candidates, recall_results, context={"query": "..."})
        top_memories = result.memories[:5]
    """

    def __init__(
        self,
        signals: list[RerankSignal] | None = None,
        recall_multiplier: int = 3,
        min_score_threshold: float = 0.1,
    ):
        """
        Args:
            signals: 重排序信号列表（默认使用 DEFAULT_SIGNALS）
            recall_multiplier: 召回倍数（最终需要 K 条，召回 K * multiplier 条）
            min_score_threshold: 最低分数阈值，低于此分数的候选被过滤
        """
        self.signals = signals or list(DEFAULT_SIGNALS)
        self.recall_multiplier = recall_multiplier
        self.min_score_threshold = min_score_threshold

    @property
    def recall_top_k(self) -> int:
        """建议的召回数量（基于 recall_multiplier）"""
        return self.recall_multiplier

    def suggest_recall_k(self, final_k: int) -> int:
        """根据最终需要的数量，计算建议的召回数量"""
        return final_k * self.recall_multiplier

    def rerank(
        self,
        memories: list[MemoryUnit],
        recall_results: list[VectorSearchResult],
        context: dict[str, Any] | None = None,
        top_k: int = 5,
    ) -> RerankResult:
        """
        对召回的记忆进行重排序

        Args:
            memories: 召回的 MemoryUnit 列表
            recall_results: 向量搜索结果（提供 vector_score）
            context: 查询上下文（query, task_type 等）
            top_k: 最终返回数量

        Returns:
            RerankResult
        """
        start = time.monotonic()
        ctx = context or {}

        # 构建 id → vector_score 映射
        score_map: dict[str, float] = {r.id: r.score for r in recall_results}

        # 构建候选列表
        candidates = [
            RerankCandidate(
                memory=mem,
                vector_score=score_map.get(getattr(mem, "entry_id", ""), 0.0),  # type: ignore[attr-defined]
            )
            for mem in memories
        ]

        # 计算每个候选的加权分数
        total_weight = sum(s.weight for s in self.signals)
        if total_weight <= 0:
            total_weight = 1.0

        for candidate in candidates:
            weighted_sum = 0.0
            for signal in self.signals:
                s = signal.score(candidate, ctx)
                candidate.signal_scores[signal.name] = s
                weighted_sum += s * signal.weight
            candidate.rerank_score = weighted_sum / total_weight

        # 过滤低分候选
        candidates = [c for c in candidates if c.rerank_score >= self.min_score_threshold]

        # 按 rerank_score 降序排序
        candidates.sort(key=lambda c: c.rerank_score, reverse=True)

        elapsed = (time.monotonic() - start) * 1000
        return RerankResult(
            candidates=candidates[:top_k],
            recall_count=len(memories),
            rerank_count=len(candidates),
            elapsed_ms=round(elapsed, 2),
        )



"""
ExecutionEngine - Agent 执行引擎

从 Agent.core 提取的核心执行循环。
负责 Agent 的迭代执行、工具调用、检查点管理。

v0.6: 消息直接写入 L1 滑动窗口，不再使用 accumulated_messages。
"""

from __future__ import annotations

import json
import logging
from typing import TYPE_CHECKING, Any

from loom.exceptions import TaskComplete
from loom.observability.metrics import LoomMetrics
from loom.runtime import Task, TaskStatus

if TYPE_CHECKING:
    from loom.agent.core import Agent

logger = logging.getLogger(__name__)


def _extract_response_from_l1(memory: Any) -> str:
    """Extract the response from L1 assistant messages.

    Scans L1 message history in reverse to find assistant text content.
    Skips tool-call-only assistant messages (no text content).
    """
    items = memory.get_message_items()
    # Collect assistant text from the last contiguous assistant turn
    parts: list[str] = []
    for item in reversed(items):
        if item.role == "assistant" and item.content:
            parts.append(str(item.content))
        elif item.role == "assistant":
            continue  # tool-call-only assistant message, skip
        else:
            if parts:
                break  # hit a non-assistant message after collecting, stop
    parts.reverse()
    return "\n".join(parts)


class ExecutionEngine:
    """
    Agent 执行引擎

    核心理念：Agent is just a for loop

    v0.6 变更：
    - 消息直接写入 L1 滑动窗口（memory.add_message）
    - 移除 accumulated_messages，L1 即对话历史
    - L1 驱逐自动触发 L2/L3 管线
    """

    def __init__(self, agent: Agent):
        self._agent = agent

    async def run(self, task: Task) -> Task:
        """
        执行任务的核心循环

        Args:
            task: 待执行的任务

        Returns:
            更新后的任务
        """
        agent = self._agent

        await agent._ensure_pending_tools_registered()
        await agent._ensure_shared_task_context(task)

        agent._current_task_id = task.taskId  # type: ignore[attr-defined]
        agent._current_session_id = task.sessionId  # type: ignore[attr-defined]
        agent._tracer.new_trace()

        try:
            # 记忆压缩
            if hasattr(agent, "memory_compactor") and agent.memory_compactor:
                current_context = await agent.context_orchestrator.build_context(
                    task.parameters.get("content", "")
                )
                compacted = await agent.memory_compactor.check_and_compact(
                    task,
                    current_context,
                    agent.context_orchestrator.max_tokens,
                )
                if compacted:
                    logger.info("Memory compaction triggered for task %s", task.taskId)

            # 加载 Skills
            task_content = task.parameters.get("content", "")
            activated_skills = await agent._load_relevant_skills(task_content)
            injected_instructions = activated_skills.get("injected_instructions", [])

            # 将用户输入写入 L1 滑动窗口
            agent.memory.add_message(
                "user",
                task_content,
                metadata={"task_id": task.taskId, "session_id": task.sessionId},
            )

            # 循环状态
            final_content = ""
            iteration = 0

            # Checkpoint resume
            start_iteration = 0
            if agent._checkpoint_mgr:
                _cp = await agent._checkpoint_mgr.load_latest(agent.node_id, task.taskId)
                if _cp:
                    start_iteration = _cp.iteration + 1
                    if _cp.memory_snapshot and hasattr(agent.memory, "restore_snapshot"):
                        agent.memory.restore_snapshot(_cp.memory_snapshot)
                    logger.info("Resuming from checkpoint iteration %d", _cp.iteration)

            try:
                for iteration in range(start_iteration, agent.max_iterations):
                    agent._metrics.increment(LoomMetrics.ITERATIONS_TOTAL)

                    # 更新预算阶段
                    agent._adaptive_budget.update_phase(
                        current_iteration=iteration,
                        max_iterations=agent.max_iterations,
                    )

                    # 获取 L1 对话历史
                    l1_messages = agent.memory.get_context_messages()

                    # 过滤 ephemeral 消息
                    filtered_messages = agent._filter_ephemeral_messages(l1_messages)

                    # 更新统一检索源的对话上下文
                    if agent._retrieval_source is not None:
                        agent._retrieval_source.set_context_messages(filtered_messages)

                    # 构建上下文（system prompt + L2/L3 + other sources）
                    messages = await agent.context_orchestrator.build_context(
                        task.parameters.get("content", "")
                    )

                    # Skills 指令注入（仅第一次迭代）
                    if injected_instructions and iteration == 0:
                        skill_instructions = "\n\n=== Available Skills ===\n\n"
                        skill_instructions += "\n\n".join(injected_instructions)
                        messages.append({"role": "system", "content": skill_instructions})

                    # 追加 L1 对话历史
                    if filtered_messages:
                        messages.extend(filtered_messages)

                    # LLM 调用
                    full_content = ""
                    tool_calls = []

                    async for chunk in agent.llm_provider.stream_chat(
                        messages,
                        tools=agent.all_tools if agent.all_tools else None,
                    ):
                        if chunk.type == "text":
                            content_str = (
                                str(chunk.content)
                                if isinstance(chunk.content, dict)
                                else chunk.content
                            )
                            full_content += content_str
                            await agent.publish_thinking(
                                content=content_str,
                                task_id=task.taskId,
                                metadata={"iteration": iteration},
                                session_id=task.sessionId,
                            )
                        elif chunk.type == "tool_call_complete":
                            if isinstance(chunk.content, dict):
                                tool_calls.append(chunk.content)
                            else:
                                try:
                                    tool_calls.append(json.loads(str(chunk.content)))
                                except (json.JSONDecodeError, TypeError):
                                    tool_calls.append(
                                        {"name": "", "arguments": {}, "content": str(chunk.content)}
                                    )
                        elif chunk.type == "error":
                            await agent._publish_event(
                                action="node.error",
                                parameters={"error": chunk.content},
                                task_id=task.taskId,
                            )

                    final_content = full_content

                    # 剥离 importance 标记
                    from loom.agent.core import _strip_importance_tag

                    clean_content, importance = _strip_importance_tag(full_content)
                    if importance is not None:
                        full_content = clean_content
                        final_content = clean_content
                        task.metadata["importance"] = importance

                    # 构建 importance metadata（传入 L1，驱逐时流入 L2）
                    _imp_meta = {"importance": importance} if importance is not None else {}

                    # 无工具调用
                    if not tool_calls:
                        if agent.require_done_tool:
                            # Record assistant text to L1, then remind to call done
                            if full_content:
                                agent.memory.add_message("assistant", full_content, metadata=_imp_meta)
                            agent.memory.add_message(
                                "system",
                                "Please call the 'done' tool to signal task completion.",
                                metadata={"ephemeral": True},
                            )
                            continue
                        else:
                            # 记录 assistant 最终回复到 L1
                            agent.memory.add_message("assistant", full_content, metadata=_imp_meta)
                            break

                    # 有工具调用 — 记录 assistant 消息到 L1
                    formatted_tool_calls = [
                        {
                            "id": tc.get("id", f"call_{i}"),
                            "type": "function",
                            "function": {
                                "name": tc.get("name", ""),
                                "arguments": (
                                    tc.get("arguments", "{}")
                                    if isinstance(tc.get("arguments"), str)
                                    else json.dumps(tc.get("arguments", {}))
                                ),
                            },
                        }
                        for i, tc in enumerate(tool_calls)
                    ]
                    agent.memory.add_message(
                        "assistant",
                        full_content or "",
                        tool_calls=formatted_tool_calls,
                        metadata=_imp_meta,
                    )

                    # 执行工具调用
                    for tool_call in tool_calls:
                        if not isinstance(tool_call, dict):
                            continue
                        tool_name = tool_call.get("name", "")
                        tool_args = tool_call.get("arguments", {})
                        if isinstance(tool_args, str):
                            try:
                                tool_args = json.loads(tool_args)
                            except json.JSONDecodeError:
                                tool_args = {}
                        if not isinstance(tool_args, dict):
                            tool_args = {}

                        agent._metrics.increment(LoomMetrics.TOOL_CALLS_TOTAL)

                        await agent.publish_tool_call(
                            tool_name=tool_name,
                            tool_args=tool_args,
                            task_id=task.taskId,
                            session_id=task.sessionId,
                        )

                        # done tool
                        if tool_name == "done":
                            from loom.tools.builtin.done import execute_done_tool

                            await execute_done_tool(tool_args)

                        # 元工具路由
                        if tool_name == "create_plan":
                            result = await agent._execute_plan(tool_args, task)
                        elif tool_name == "delegate_task":
                            result = await agent._auto_delegate(tool_args, task)
                        elif tool_name == "delegate_to_agent":
                            target_agent = tool_args.get("target_agent", "")
                            subtask = tool_args.get("subtask", "")
                            result = await agent._execute_delegate_task(
                                target_agent,
                                subtask,
                                task.taskId,
                                session_id=task.sessionId,
                            )
                        else:
                            result = await agent._tool_router.route(tool_name, tool_args)

                        await agent.publish_tool_result(
                            tool_name=tool_name,
                            result=result,
                            task_id=task.taskId,
                            session_id=task.sessionId,
                        )

                        # 记录工具结果到 L1
                        agent.memory.add_message(
                            "tool",
                            result,
                            tool_call_id=tool_call.get("id", ""),
                            tool_name=tool_name,
                        )

                    # 保存检查点
                    if agent._checkpoint_mgr:
                        from loom.runtime.checkpoint import CheckpointData

                        _cp_data = CheckpointData(
                            agent_id=agent.node_id,
                            task_id=task.taskId,
                            iteration=iteration,
                            max_iterations=agent.max_iterations,
                            agent_state={"final_content": final_content},
                            memory_snapshot=(
                                agent.memory.export_snapshot()
                                if hasattr(agent.memory, "export_snapshot")
                                else {}
                            ),
                            tool_history=[],  # L1 已包含完整历史
                            context_metadata={"session_id": task.sessionId},
                        )
                        await agent._checkpoint_mgr.save(_cp_data)

            except TaskComplete as e:
                task.status = TaskStatus.COMPLETED
                # Response comes from L1 assistant messages, not done.message
                response = _extract_response_from_l1(agent.memory)
                task.result = {
                    "content": response or e.message or "",
                    "message": e.message,
                    "output": e.output,
                    "completed_explicitly": True,
                }
                await agent._self_evaluate(task)
                return task

            # 循环正常结束 — extract response from L1
            response = _extract_response_from_l1(agent.memory)
            if not response:
                # Fallback: tool outputs from L1
                items = agent.memory.get_message_items()
                tool_outputs = [
                    str(item.content)
                    for item in items
                    if item.role == "tool" and item.content
                ]
                if tool_outputs:
                    response = "\n".join(tool_outputs[-5:])

            task.status = TaskStatus.COMPLETED
            task.result = {
                "content": response or "",
                "completed_explicitly": False,
                "iterations": iteration + 1,
            }

            await agent._self_evaluate(task)

            return task
        finally:
            # Trace 摘要
            _trace_summary = agent._tracer.get_trace_summary()
            logger.debug(
                "Trace summary: spans=%d, duration=%.1fms, errors=%d",
                _trace_summary["span_count"],
                _trace_summary["total_duration_ms"],
                _trace_summary["error_count"],
            )

            # Metrics 导出
            if agent._metrics_exporters:
                _snapshot = agent._metrics.snapshot()
                for _mexp in agent._metrics_exporters:
                    try:
                        _mexp.export(_snapshot)
                    except Exception:
                        logger.debug("Metrics export failed", exc_info=True)

            # 清理任务上下文
            if hasattr(agent, "_current_task_id"):
                del agent._current_task_id
            if hasattr(agent, "_current_session_id"):
                del agent._current_session_id



"""
Context Collector - 上下文收集器

协调多个上下文源，按预算收集上下文块。
"""

import asyncio
from typing import TYPE_CHECKING

from loom.context.block import ContextBlock
from loom.context.budget import BudgetAllocation
from loom.context.source import ContextSource

if TYPE_CHECKING:
    from loom.memory.tokenizer import TokenCounter


class ContextCollector:
    """
    上下文收集器

    职责：
    1. 协调多个上下文源
    2. 按预算分配收集上下文
    3. 合并和排序结果
    """

    def __init__(
        self,
        sources: list[ContextSource],
        token_counter: "TokenCounter",
    ):
        """
        初始化收集器

        Args:
            sources: 上下文源列表
            token_counter: Token 计数器
        """
        self.sources = sources
        self.token_counter = token_counter
        self._source_map = {s.source_name: s for s in sources}

    async def collect(
        self,
        query: str,
        allocation: BudgetAllocation,
        min_relevance: float = 0.5,
    ) -> list[ContextBlock]:
        """
        从所有源收集上下文

        Args:
            query: 当前任务内容
            allocation: 预算分配
            min_relevance: 最低相关性阈值

        Returns:
            按 priority 降序排列的 ContextBlock 列表
        """
        # 并行从各源收集
        tasks = []
        for source in self.sources:
            budget = allocation.get(source.source_name)
            if budget > 0:
                tasks.append(self._collect_from_source(source, query, budget, min_relevance))

        # 等待所有源完成
        results = await asyncio.gather(*tasks, return_exceptions=True)

        # 合并结果
        all_blocks: list[ContextBlock] = []
        for result in results:
            if isinstance(result, list):
                all_blocks.extend(result)
            # 忽略异常，继续处理其他源的结果

        # 按 priority 降序排序
        all_blocks.sort(key=lambda b: b.priority, reverse=True)

        return all_blocks

    async def _collect_from_source(
        self,
        source: ContextSource,
        query: str,
        budget: int,
        min_relevance: float,
    ) -> list[ContextBlock]:
        """从单个源收集"""
        return await source.collect(
            query=query,
            token_budget=budget,
            token_counter=self.token_counter,
            min_relevance=min_relevance,
        )

    async def collect_from_specific(
        self,
        query: str,
        source_budgets: dict[str, int],
        min_relevance: float = 0.5,
    ) -> list[ContextBlock]:
        """
        从指定源收集

        Args:
            query: 当前任务内容
            source_budgets: 源名称 -> 预算
            min_relevance: 最低相关性阈值

        Returns:
            按 priority 降序排列的 ContextBlock 列表
        """
        tasks = []
        for source_name, budget in source_budgets.items():
            source = self._source_map.get(source_name)
            if source and budget > 0:
                tasks.append(self._collect_from_source(source, query, budget, min_relevance))

        results = await asyncio.gather(*tasks, return_exceptions=True)

        all_blocks: list[ContextBlock] = []
        for result in results:
            if isinstance(result, list):
                all_blocks.extend(result)

        all_blocks.sort(key=lambda b: b.priority, reverse=True)
        return all_blocks



"""
事件动作类型定义

提供类型安全的事件动作枚举，替代字符串键值。
"""

from enum import StrEnum


class TaskAction(StrEnum):
    """任务动作类型（类型安全）"""

    EXECUTE = "execute"
    CANCEL = "cancel"
    QUERY = "query"
    STREAM = "stream"


class MemoryAction(StrEnum):
    """记忆动作类型（类型安全）"""

    READ = "read_memory"
    WRITE = "write_memory"
    SEARCH = "search_memory"
    SYNC = "sync_memory"


class AgentAction(StrEnum):
    """Agent动作类型（类型安全）"""

    START = "start_agent"
    STOP = "stop_agent"
    STATUS = "agent_status"
    HEARTBEAT = "agent_heartbeat"


class KnowledgeAction(StrEnum):
    """知识检索动作类型（类型安全）"""

    SEARCH = "knowledge.search"  # 主动搜索
    SEARCH_RESULT = "knowledge.result"  # 搜索结果



"""
向量知识库实现

基于向量搜索的知识库，使用embedding进行语义匹配
"""

from typing import Any

from loom.memory.vector_store import EmbeddingProvider, VectorStoreProvider
from loom.providers.knowledge.base import KnowledgeBaseProvider, KnowledgeItem


class VectorKnowledgeBase(KnowledgeBaseProvider):
    """
    基于向量搜索的知识库实现

    特点：
    - 使用embedding向量进行语义搜索
    - 支持自然语言查询
    - 高召回率和相关度
    """

    def __init__(
        self,
        embedding_provider: EmbeddingProvider,
        vector_store: VectorStoreProvider,
    ):
        """
        初始化向量知识库

        Args:
            embedding_provider: Embedding提供者
            vector_store: 向量存储提供者
        """
        self.embedding_provider = embedding_provider
        self.vector_store = vector_store
        self.items: dict[str, KnowledgeItem] = {}

    async def add_item(self, item: KnowledgeItem) -> None:
        """
        添加知识条目（自动生成embedding）

        Args:
            item: 知识条目
        """
        # 生成embedding
        embedding = await self.embedding_provider.embed(item.content)

        # 存储到vector store
        await self.vector_store.add(
            id=item.id,
            embedding=embedding,
            metadata={"source": item.source},
        )

        # 保存原始数据
        self.items[item.id] = item

    async def query(
        self,
        query: str,
        limit: int = 5,
        _filters: dict[str, Any] | None = None,
    ) -> list[KnowledgeItem]:
        """
        向量语义搜索

        Args:
            query: 查询文本
            limit: 返回结果数量
            _filters: 过滤条件（暂未使用）

        Returns:
            匹配的知识条目列表（按相关度排序）
        """
        # 查询向量化
        query_embedding = await self.embedding_provider.embed(query)

        # 向量搜索
        results = await self.vector_store.search(query_embedding, top_k=limit)

        # 转换为KnowledgeItem
        knowledge_items = []
        for result in results:
            if result.id in self.items:
                item = self.items[result.id]
                item.relevance = result.score  # 设置相关度分数
                knowledge_items.append(item)

        return knowledge_items

    async def get_by_id(self, knowledge_id: str) -> KnowledgeItem | None:
        """
        根据ID获取知识条目

        Args:
            knowledge_id: 知识条目ID

        Returns:
            知识条目，如果不存在则返回None
        """
        return self.items.get(knowledge_id)



"""
分形记忆系统 (Fractal Memory System)

基于科赫雪花的分形理念：在有限时间距离下实现无限思考。

核心组件：
- MemoryScope: 记忆作用域枚举
- MemoryAccessPolicy: 记忆访问策略
- MemoryEntry: 记忆条目数据结构
- FractalMemory: 分形记忆管理器

设计原则：
1. 最小必要原则 - 子节点只接收完成任务所需的最小上下文
2. 分层可见性 - 不同层级的记忆有不同的可见范围
3. 按需加载 - 上下文和记忆按需传递，而非全量复制
4. 双向流动 - 信息可以从父到子，也可以从子到父
5. 冲突可解 - 提供多种策略解决记忆冲突
"""

from dataclasses import dataclass, field
from enum import StrEnum
from typing import TYPE_CHECKING, Any, Optional

if TYPE_CHECKING:
    from loom.memory.core import LoomMemory


class MemoryScope(StrEnum):
    """
    记忆作用域

    定义记忆在分形节点树中的可见性和访问权限。
    """

    LOCAL = "local"  # 节点私有，不共享
    SHARED = "shared"  # 父子双向共享
    INHERITED = "inherited"  # 从父节点继承（只读）
    GLOBAL = "global"  # 全局共享（所有节点）


@dataclass
class MemoryAccessPolicy:
    """
    记忆访问策略

    定义每种作用域的访问权限和传播规则。
    """

    scope: MemoryScope
    readable: bool  # 是否可读
    writable: bool  # 是否可写
    propagate_up: bool  # 是否向上传播（子→父）
    propagate_down: bool  # 是否向下传播（父→子）


# 预定义的访问策略
ACCESS_POLICIES = {
    MemoryScope.LOCAL: MemoryAccessPolicy(
        scope=MemoryScope.LOCAL,
        readable=True,
        writable=True,
        propagate_up=False,
        propagate_down=False,
    ),
    MemoryScope.SHARED: MemoryAccessPolicy(
        scope=MemoryScope.SHARED,
        readable=True,
        writable=True,
        propagate_up=True,
        propagate_down=True,
    ),
    MemoryScope.INHERITED: MemoryAccessPolicy(
        scope=MemoryScope.INHERITED,
        readable=True,
        writable=False,  # 只读
        propagate_up=False,
        propagate_down=True,
    ),
    MemoryScope.GLOBAL: MemoryAccessPolicy(
        scope=MemoryScope.GLOBAL,
        readable=True,
        writable=True,
        propagate_up=True,
        propagate_down=True,
    ),
}


@dataclass
class MemoryEntry:
    """
    记忆条目

    存储单个记忆项的完整信息，包括内容、作用域、版本控制等元数据。
    """

    id: str  # 唯一标识
    content: Any  # 记忆内容
    scope: MemoryScope  # 作用域
    version: int = 1  # 版本号（用于冲突检测）
    created_by: str = ""  # 创建者节点ID
    updated_by: str = ""  # 最后更新者节点ID
    parent_version: int | None = None  # 父版本号（用于追踪）
    metadata: dict[str, Any] = field(default_factory=dict)  # 元数据

    def __post_init__(self):
        """初始化后处理：确保metadata不为None"""
        if self.metadata is None:
            self.metadata = {}


class FractalMemory:
    """
    分形记忆管理器

    职责：
    - 管理不同作用域的记忆
    - 处理父子节点间的记忆共享
    - 提供统一的读写接口
    - 使用LoomMemory作为底层存储
    """

    def __init__(
        self,
        node_id: str,
        parent_memory: Optional["FractalMemory"] = None,
        base_memory: Optional["LoomMemory"] = None,
    ):
        """
        初始化分形记忆管理器

        Args:
            node_id: 节点唯一标识
            parent_memory: 父节点的记忆管理器（用于建立父子关系）
            base_memory: 底层LoomMemory存储（如果为None，将在需要时创建）
        """
        self.node_id = node_id
        self.parent_memory = parent_memory
        self.base_memory = base_memory

        # 按作用域组织的记忆索引（轻量级，只存储元数据）
        self._memory_by_scope: dict[MemoryScope, dict[str, MemoryEntry]] = {
            scope: {} for scope in MemoryScope
        }

    async def write(
        self, entry_id: str, content: Any, scope: MemoryScope = MemoryScope.LOCAL
    ) -> MemoryEntry:
        """
        写入记忆

        Args:
            entry_id: 记忆ID
            content: 记忆内容
            scope: 作用域

        Returns:
            创建的记忆条目

        Raises:
            PermissionError: 如果作用域不可写
        """
        # 检查写权限
        policy = ACCESS_POLICIES[scope]
        if not policy.writable:
            raise PermissionError(f"Scope {scope} is read-only")

        # 如果已存在，更新并递增版本
        existing = self._memory_by_scope[scope].get(entry_id)
        if existing:
            existing.parent_version = existing.version
            existing.version += 1
            existing.content = content
            existing.updated_by = self.node_id
            return existing

        # 创建记忆条目
        entry = MemoryEntry(
            id=entry_id,
            content=content,
            scope=scope,
            created_by=self.node_id,
            updated_by=self.node_id,
        )

        # 存储到对应作用域
        self._memory_by_scope[scope][entry_id] = entry

        return entry

    async def write_entry(self, entry: MemoryEntry) -> MemoryEntry:
        """
        写入完整记忆条目（保留版本/元数据）

        Args:
            entry: MemoryEntry对象

        Returns:
            写入后的记忆条目
        """
        # 检查写权限
        policy = ACCESS_POLICIES[entry.scope]
        if not policy.writable:
            raise PermissionError(f"Scope {entry.scope} is read-only")

        # 直接写入（覆盖）
        self._memory_by_scope[entry.scope][entry.id] = entry
        return entry

    async def read(
        self,
        entry_id: str,
        search_scopes: list[MemoryScope] | None = None,
    ) -> MemoryEntry | None:
        """
        读取记忆

        Args:
            entry_id: 记忆ID
            search_scopes: 搜索的作用域列表（None表示搜索所有）

        Returns:
            记忆条目，如果不存在返回None
        """
        if search_scopes is None:
            search_scopes = list(MemoryScope)

        # 按优先级搜索：LOCAL > SHARED > INHERITED > GLOBAL
        for scope in search_scopes:
            if entry_id in self._memory_by_scope[scope]:
                return self._memory_by_scope[scope][entry_id]

        # 如果是INHERITED作用域，尝试从父节点读取
        if MemoryScope.INHERITED in search_scopes and self.parent_memory:
            parent_entry = await self.parent_memory.read(
                entry_id,
                search_scopes=[MemoryScope.SHARED, MemoryScope.GLOBAL, MemoryScope.INHERITED],
            )
            if parent_entry:
                # 创建只读副本
                inherited_entry = MemoryEntry(
                    id=parent_entry.id,
                    content=parent_entry.content,
                    scope=MemoryScope.INHERITED,
                    version=parent_entry.version,
                    created_by=parent_entry.created_by,
                    updated_by=parent_entry.updated_by,
                    parent_version=parent_entry.version,
                )
                # 缓存到本地
                self._memory_by_scope[MemoryScope.INHERITED][entry_id] = inherited_entry
                return inherited_entry

        return None

    async def list_by_scope(self, scope: MemoryScope) -> list[MemoryEntry]:
        """
        列出指定作用域的所有记忆

        Args:
            scope: 记忆作用域

        Returns:
            该作用域下的所有记忆条目列表
        """
        return list(self._memory_by_scope[scope].values())



"""
AgentBuilder - 链式构建器

提供流畅的链式API风格创建Agent。

从 core.py 拆分，遵循单一职责原则。
"""

from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from loom.config.context import ContextConfig
    from loom.config.memory import MemoryConfig
    from loom.config.tool import ToolConfig
    from loom.memory.compaction import CompactionConfig
    from loom.providers.llm.interface import LLMProvider
    from loom.runtime.session_lane import SessionIsolationMode
    from loom.security.tool_policy import ToolPolicy

    from .core import Agent


class AgentBuilder:
    """
    Agent构建器 - 支持流畅的链式调用

    提供类似llama-index的流畅API风格。

    Examples:
        >>> agent = (Agent.builder(llm)
        ...     .with_system_prompt("你是一个AI助手")
        ...     .with_tools([...])
        ...     .with_memory(max_tokens=4000)
        ...     .build())
    """

    def __init__(self, llm: "LLMProvider"):
        """初始化构建器"""
        self.llm = llm
        self.config: dict[str, Any] = {}

    def with_system_prompt(self, prompt: str) -> "AgentBuilder":
        """设置系统提示词"""
        self.config["system_prompt"] = prompt
        return self

    def with_tools(self, tools: list[dict[str, Any]]) -> "AgentBuilder":
        """设置工具列表"""
        self.config["tools"] = tools
        return self

    def with_memory(self, max_tokens: int = 4000) -> "AgentBuilder":
        """配置上下文窗口（token上限）"""
        self.config["max_context_tokens"] = max_tokens
        return self

    def with_context_window(self, max_tokens: int = 4000) -> "AgentBuilder":
        """配置上下文窗口（别名）"""
        return self.with_memory(max_tokens)

    def with_memory_config(self, memory_config: "MemoryConfig | dict[str, Any]") -> "AgentBuilder":
        """配置记忆系统（MemoryConfig 或 dict）"""
        self.config["memory_config"] = memory_config
        return self

    def with_context_budget(self, budget_config: Any) -> "AgentBuilder":
        """配置上下文预算（BudgetConfig 或 dict）"""
        self.config["context_budget_config"] = budget_config
        return self

    def with_context_config(
        self, context_config: "ContextConfig | dict[str, Any]"
    ) -> "AgentBuilder":
        """配置上下文流动（ContextConfig 或 dict）"""
        self.config["context_config"] = context_config
        return self

    def with_knowledge_base(self, knowledge_base: Any) -> "AgentBuilder":
        """设置知识库"""
        self.config["knowledge_base"] = knowledge_base
        return self

    def with_event_bus(self, event_bus: Any) -> "AgentBuilder":
        """设置事件总线"""
        self.config["event_bus"] = event_bus
        return self

    def with_iterations(self, max_iterations: int) -> "AgentBuilder":
        """设置最大迭代次数"""
        self.config["max_iterations"] = max_iterations
        return self

    def with_session_isolation(self, mode: "SessionIsolationMode | str") -> "AgentBuilder":
        """设置 Session 隔离模式（strict/advisory/none）"""
        self.config["session_isolation"] = mode
        return self

    def with_compaction(
        self, compaction_config: "CompactionConfig | dict[str, Any]"
    ) -> "AgentBuilder":
        """设置记忆压缩配置"""
        self.config["compaction_config"] = compaction_config
        return self

    def with_tool_policy(self, tool_policy: "ToolPolicy") -> "AgentBuilder":
        """设置工具权限策略"""
        self.config["tool_policy"] = tool_policy
        return self

    def with_skills(self, skills: list[str]) -> "AgentBuilder":
        """设置技能列表"""
        self.config["skills"] = skills
        return self

    def with_skills_dir(self, skills_dir: str | Path | list[str | Path]) -> "AgentBuilder":
        """设置 Skills 目录（SKILL.md 包）"""
        self.config["skills_dir"] = skills_dir
        return self

    def with_skill_loaders(self, skill_loaders: list[Any]) -> "AgentBuilder":
        """设置 Skills 加载器列表"""
        self.config["skill_loaders"] = skill_loaders
        return self

    def with_tool_config(self, tool_config: "ToolConfig | dict[str, Any]") -> "AgentBuilder":
        """设置工具聚合配置（v0.5.1 渐进式披露）"""
        self.config["tool_config"] = tool_config
        return self

    def with_capabilities(self, capabilities: Any) -> "AgentBuilder":
        """设置能力注册表（CapabilityRegistry）"""
        self.config["capabilities"] = capabilities
        return self

    def with_knowledge_config(
        self, max_items: int = 3, relevance_threshold: float = 0.7
    ) -> "AgentBuilder":
        """配置知识库查询参数"""
        self.config["knowledge_max_items"] = max_items
        self.config["knowledge_relevance_threshold"] = relevance_threshold
        return self

    def with_done_tool(self, require: bool = True) -> "AgentBuilder":
        """设置是否要求显式调用done工具"""
        self.config["require_done_tool"] = require
        return self

    def build(self) -> "Agent":
        """构建Agent实例"""
        from .core import Agent

        return Agent.create(self.llm, **self.config)



"""
Loom Framework - 统一对外API

基于六大公理系统的AI Agent框架。

核心导出：
- 协议层（A1）
- 事件层（A2）
- 分形层（A3）
- 记忆层（A4）
- 流式API
- 配置层
- 运行时
- 安全层
"""

# A1: 统一接口公理 - 协议层
# 配置层
# Stream API - 分形流式观测
from loom.api.stream_api import (
    FractalEvent,
    FractalStreamAPI,
    OutputStrategy,
    StreamAPI,
)

# Version API - 版本管理
from loom.api.version import (
    ChangeType,
    VersionAPI,
    VersionInfo,
    get_version,
    get_version_info,
)
from loom.config import ContextConfig
from loom.config.context import BudgetConfig
from loom.config.tool import ToolConfig

# A2: 事件主权公理 - 事件层
from loom.events import (
    ContextController,
    EventBus,
    Session,
    SessionStatus,
    SSEFormatter,
)

# A3: 分形自相似公理 - 分形层
from loom.fractal import NodeContainer

# A4: 记忆层次公理 - 记忆层
from loom.memory import (
    MemoryManager,
    MemoryQuery,
    MemoryTier,
    MemoryType,
    MessageItem,
)
from loom.memory.compaction import CompactionConfig

# A1: 统一接口公理 - 协议层（Task 已迁移到 runtime）
# Runtime
from loom.runtime import Dispatcher, Interceptor, InterceptorChain, Task, TaskStatus
from loom.runtime.session_lane import SessionIsolationMode

# Security
from loom.security import BlacklistPolicy, ToolPolicy, WhitelistPolicy

__version__ = "0.5.7"

__all__ = [
    # Protocol (Task moved to Runtime)
    "Task",
    "TaskStatus",
    # Events
    "EventBus",
    "Session",
    "SessionStatus",
    "ContextController",
    "SSEFormatter",
    # Fractal
    "NodeContainer",
    # Memory
    "MemoryManager",
    "MessageItem",
    "MemoryTier",
    "MemoryType",
    "MemoryQuery",
    "ContextConfig",
    "ToolConfig",
    "CompactionConfig",
    "BudgetConfig",
    # Runtime
    "Dispatcher",
    "Interceptor",
    "InterceptorChain",
    "SessionIsolationMode",
    # Security
    "ToolPolicy",
    "WhitelistPolicy",
    "BlacklistPolicy",
    # Stream API
    "StreamAPI",
    "FractalStreamAPI",
    "FractalEvent",
    "OutputStrategy",
    # Version API
    "VersionAPI",
    "VersionInfo",
    "ChangeType",
    "get_version",
    "get_version_info",
]



"""
图优先检索策略
"""

from __future__ import annotations

import math
import time
from typing import TYPE_CHECKING, Any

from loom.providers.knowledge.rag.models.chunk import TextChunk
from loom.providers.knowledge.rag.models.result import RetrievalResult
from loom.providers.knowledge.rag.retrievers.graph import GraphRetriever
from loom.providers.knowledge.rag.retrievers.vector import VectorRetriever
from loom.providers.knowledge.rag.strategies.base import (
    RetrievalStrategy,
    StrategyType,
)

if TYPE_CHECKING:
    from loom.observability.metrics import LoomMetrics
    from loom.observability.tracing import LoomTracer


class GraphFirstStrategy(RetrievalStrategy):
    """
    图优先策略

    核心流程：
    1. 图检索获取结构相关内容
    2. 语义排序精选结果
    3. 降级：图检索无结果时使用纯向量检索
    """

    def __init__(
        self,
        graph_retriever: GraphRetriever,
        vector_retriever: VectorRetriever,
        n_hop: int = 2,
        tracer: LoomTracer | None = None,
        metrics: LoomMetrics | None = None,
    ):
        self.graph_retriever = graph_retriever
        self.vector_retriever = vector_retriever
        self.n_hop = n_hop
        self.tracer = tracer
        self.metrics = metrics

    @property
    def strategy_type(self) -> StrategyType:
        return StrategyType.GRAPH_FIRST

    async def retrieve(
        self,
        query: str,
        limit: int = 10,
        **_kwargs: Any,
    ) -> RetrievalResult:
        """图优先 + 语义排序"""
        t0 = time.monotonic()

        # 1. 图检索
        entities, relations, graph_chunks = await self.graph_retriever.retrieve(
            query, n_hop=self.n_hop, limit=limit * 2
        )
        t_graph = time.monotonic()

        if not graph_chunks:
            # 降级：纯向量检索
            vector_results = await self.vector_retriever.retrieve(query, limit)
            t_end = time.monotonic()
            self._record_metrics(
                graph_count=0,
                result_count=len(vector_results),
                fallback=True,
                total_ms=(t_end - t0) * 1000,
            )
            return RetrievalResult(
                chunks=[c for c, _ in vector_results],
                scores={c.id: s for c, s in vector_results},
            )

        # 2. 语义排序
        scored_chunks = await self._semantic_rerank(query, graph_chunks)

        # 3. 取 Top-K
        sorted_chunks = sorted(scored_chunks, key=lambda x: x[1], reverse=True)[:limit]
        t_end = time.monotonic()

        self._record_metrics(
            graph_count=len(graph_chunks),
            result_count=len(sorted_chunks),
            fallback=False,
            graph_ms=(t_graph - t0) * 1000,
            total_ms=(t_end - t0) * 1000,
        )

        return RetrievalResult(
            chunks=[c for c, _ in sorted_chunks],
            entities=entities,
            relations=relations,
            scores={c.id: s for c, s in sorted_chunks},
        )

    async def _semantic_rerank(
        self,
        query: str,
        chunks: list[TextChunk],
    ) -> list[tuple[TextChunk, float]]:
        """语义重排序"""
        query_embedding = await self.vector_retriever.embedding_provider.embed(query)

        results = []
        for chunk in chunks:
            if chunk.embedding:
                score = self._cosine_similarity(query_embedding, chunk.embedding)
            else:
                score = 0.0
            results.append((chunk, score))

        return results

    @staticmethod
    def _cosine_similarity(vec1: list[float], vec2: list[float]) -> float:
        """计算余弦相似度"""
        if len(vec1) != len(vec2):
            return 0.0
        dot_product = sum(a * b for a, b in zip(vec1, vec2, strict=False))
        norm1 = math.sqrt(sum(a * a for a in vec1))
        norm2 = math.sqrt(sum(b * b for b in vec2))
        if norm1 == 0 or norm2 == 0:
            return 0.0
        return dot_product / (norm1 * norm2)

    def _record_metrics(
        self,
        graph_count: int,
        result_count: int,
        fallback: bool,
        total_ms: float,
        graph_ms: float = 0.0,
    ) -> None:
        """记录检索指标到观测体系"""
        if self.tracer:
            span = self.tracer.current_span
            if span:
                span.set_attribute("retrieval.strategy", "graph_first")
                span.set_attribute("retrieval.graph_count", graph_count)
                span.set_attribute("retrieval.result_count", result_count)
                span.set_attribute("retrieval.fallback_to_vector", fallback)
                span.set_attribute("retrieval.graph_ms", round(graph_ms, 2))
                span.set_attribute("retrieval.total_ms", round(total_ms, 2))

        if self.metrics:
            from loom.observability.metrics import LoomMetrics

            self.metrics.increment(LoomMetrics.KNOWLEDGE_SEARCH_TOTAL)
            self.metrics.observe(LoomMetrics.KNOWLEDGE_SEARCH_LATENCY, total_ms)
            self.metrics.set_gauge(LoomMetrics.KNOWLEDGE_RESULTS_COUNT, result_count)



"""
Session Lane - 同 session 串行控制

提供三种隔离模式：
- strict: 严格串行（默认）
- advisory: 仅警告
- none: 不控制

符合 Loom 框架原则：提供机制，应用选择策略
"""

import asyncio
import logging
from enum import StrEnum
from typing import Any

from loom.events.actions import KnowledgeAction
from loom.runtime.interceptor import Interceptor
from loom.runtime.task import Task

logger = logging.getLogger(__name__)


class SessionIsolationMode(StrEnum):
    """Session 隔离模式"""

    STRICT = "strict"  # 严格串行（默认）
    ADVISORY = "advisory"  # 仅警告
    NONE = "none"  # 不控制


class SessionLaneInterceptor(Interceptor):
    """
    同 session 串行控制拦截器

    使用 asyncio.Lock 确保同一 session_id 的任务串行执行，
    避免上下文交叉破坏记忆一致性。
    """

    def __init__(self, mode: SessionIsolationMode = SessionIsolationMode.STRICT):
        """
        初始化 Session Lane 拦截器

        Args:
            mode: 隔离模式（strict/advisory/none）
        """
        self.mode = mode
        self._locks: dict[tuple[str, str], asyncio.Lock] = {}
        self._lock_registry = asyncio.Lock()
        self._task_locks: dict[str, asyncio.Lock] = {}  # taskId -> lock 映射
        self._lock_usage_count: dict[tuple[str, str], int] = {}  # 锁使用计数

    async def before(self, task: Task) -> Task:
        """
        执行前检查并获取锁

        Args:
            task: 任务

        Returns:
            Task: 原任务（可能被修改）
        """
        # 无 session_id 则放行
        if not task.sessionId:
            return task

        # KnowledgeAction 是 Agent 内部子操作，不参与 session 串行控制
        if task.action in (KnowledgeAction.SEARCH, KnowledgeAction.SEARCH_RESULT):
            return task

        # 使用 task.taskId 作为节点标识（因为没有 context）
        node_id = "default"
        lock_key = (node_id, task.sessionId)

        # 获取或创建锁
        async with self._lock_registry:
            if lock_key not in self._locks:
                self._locks[lock_key] = asyncio.Lock()
                self._lock_usage_count[lock_key] = 0
            lock = self._locks[lock_key]
            self._lock_usage_count[lock_key] += 1

        # 根据模式处理
        if self.mode == SessionIsolationMode.STRICT:
            # 严格模式：阻塞等待
            await lock.acquire()
            self._task_locks[task.taskId] = lock
            # 保存 lock_key 用于 after 清理
            task.metadata["_session_lane_lock_key"] = lock_key
            logger.debug(f"Session lane acquired for session {task.sessionId}")

        elif self.mode == SessionIsolationMode.ADVISORY:
            # 警告模式：检测并发但不阻塞
            if lock.locked():
                logger.warning(
                    f"Concurrent execution detected for session {task.sessionId}. "
                    f"Consider using strict mode."
                )

        # NONE 模式：什么都不做

        return task

    async def after(self, task: Task) -> Task:
        """
        执行后释放锁

        Args:
            task: 任务

        Returns:
            Task: 原任务（可能被修改）
        """
        lock = self._task_locks.pop(task.taskId, None)
        if lock and lock.locked():
            lock.release()
            logger.debug(f"Session lane released for session {task.sessionId}")

        # 清理锁使用计数
        lock_key = task.metadata.pop("_session_lane_lock_key", None)
        if lock_key:
            async with self._lock_registry:
                if lock_key in self._lock_usage_count:
                    self._lock_usage_count[lock_key] -= 1
                    # 当没有任务使用该锁时，清理它
                    if self._lock_usage_count[lock_key] <= 0:
                        self._locks.pop(lock_key, None)
                        self._lock_usage_count.pop(lock_key, None)

        return task

    # ==================== 状态检查方法 ====================

    @property
    def active_task_ids(self) -> list[str]:
        """当前持有锁的任务ID列表"""
        return list(self._task_locks.keys())

    @property
    def active_locks_count(self) -> int:
        """当前活跃的锁数量"""
        return len(self._locks)

    def get_lock_stats(self) -> dict[str, Any]:
        """获取锁统计信息"""
        return {
            "active_locks": len(self._locks),
            "active_tasks": len(self._task_locks),
            "lock_usage": dict(self._lock_usage_count),
        }



"""
OTEL Exporters - OpenTelemetry 协议导出器

通过 OTLP/HTTP 协议导出 Span 和 Metrics，无需 opentelemetry-sdk 依赖。
失败时静默降级到日志。
"""

from __future__ import annotations

import json
import logging
import sys
import time
import urllib.error
import urllib.request
from typing import Any

from loom.observability.tracing import Span, SpanExporter

logger = logging.getLogger(__name__)


class OTLPSpanExporter(SpanExporter):
    """
    通过 OTLP/HTTP 导出 Span 到 Jaeger / Tempo / OTEL Collector

    用法：
        exporter = OTLPSpanExporter(endpoint="http://localhost:4318/v1/traces")
        tracer.add_exporter(exporter)
    """

    def __init__(
        self,
        endpoint: str = "http://localhost:4318/v1/traces",
        headers: dict[str, str] | None = None,
        service_name: str = "loom-agent",
        timeout: float = 5.0,
    ):
        self._endpoint = endpoint
        self._headers = headers or {}
        self._service_name = service_name
        self._timeout = timeout

    def export(self, span: Span) -> None:
        payload = self._to_otlp(span)
        self._send(payload)

    def _to_otlp(self, span: Span) -> dict[str, Any]:
        """Loom Span → OTLP ResourceSpans JSON"""
        otlp_span: dict[str, Any] = {
            "traceId": span.trace_id.ljust(32, "0"),
            "spanId": span.span_id.ljust(16, "0"),
            "name": span.name,
            "kind": 1,  # SPAN_KIND_INTERNAL
            "startTimeUnixNano": str(int(span.start_time * 1e9)),
            "endTimeUnixNano": str(int(span.end_time * 1e9)) if span.end_time > 0 else "0",
            "attributes": [
                {"key": k, "value": _to_otlp_value(v)} for k, v in span.attributes.items()
            ],
            "status": {
                "code": 2 if span.status == "error" else 1,  # ERROR=2, OK=1
                "message": span.error_message,
            },
        }
        if span.parent_span_id:
            otlp_span["parentSpanId"] = span.parent_span_id.ljust(16, "0")

        if span.events:
            otlp_span["events"] = [
                {
                    "timeUnixNano": str(int(e.get("timestamp", 0) * 1e9)),
                    "name": e.get("name", ""),
                    "attributes": [
                        {"key": k, "value": _to_otlp_value(v)}
                        for k, v in e.get("attributes", {}).items()
                    ],
                }
                for e in span.events
            ]

        return {
            "resourceSpans": [
                {
                    "resource": {
                        "attributes": [
                            {"key": "service.name", "value": {"stringValue": self._service_name}},
                        ],
                    },
                    "scopeSpans": [
                        {
                            "scope": {"name": "loom.observability"},
                            "spans": [otlp_span],
                        }
                    ],
                }
            ],
        }

    def _send(self, payload: dict[str, Any]) -> None:
        try:
            data = json.dumps(payload).encode("utf-8")
            req = urllib.request.Request(
                self._endpoint,
                data=data,
                headers={
                    "Content-Type": "application/json",
                    **self._headers,
                },
                method="POST",
            )
            urllib.request.urlopen(req, timeout=self._timeout)
        except Exception:
            logger.debug("OTLP span export failed", exc_info=True)


class OTLPMetricsExporter:
    """
    通过 OTLP/HTTP 导出 Metrics 快照

    用法：
        exporter = OTLPMetricsExporter()
        exporter.export(metrics.snapshot())
    """

    def __init__(
        self,
        endpoint: str = "http://localhost:4318/v1/metrics",
        headers: dict[str, str] | None = None,
        service_name: str = "loom-agent",
        timeout: float = 5.0,
    ):
        self._endpoint = endpoint
        self._headers = headers or {}
        self._service_name = service_name
        self._timeout = timeout

    def export(self, snapshot: dict[str, Any]) -> None:
        payload = self._to_otlp(snapshot)
        self._send(payload)

    def _to_otlp(self, snapshot: dict[str, Any]) -> dict[str, Any]:
        """LoomMetrics.snapshot() → OTLP Metrics JSON"""
        now_ns = str(int(time.time() * 1e9))
        metrics: list[dict[str, Any]] = []

        # Counters → Sum
        for name, value in snapshot.get("counters", {}).items():
            metrics.append(
                {
                    "name": name,
                    "sum": {
                        "dataPoints": [
                            {
                                "asDouble": value,
                                "timeUnixNano": now_ns,
                                "isMonotonic": True,
                                "aggregationTemporality": 2,  # CUMULATIVE
                            }
                        ],
                    },
                }
            )

        # Gauges → Gauge
        for name, value in snapshot.get("gauges", {}).items():
            metrics.append(
                {
                    "name": name,
                    "gauge": {
                        "dataPoints": [
                            {
                                "asDouble": value,
                                "timeUnixNano": now_ns,
                            }
                        ],
                    },
                }
            )

        # Histograms → Summary (simplified)
        for name, stats in snapshot.get("histograms", {}).items():
            metrics.append(
                {
                    "name": name,
                    "summary": {
                        "dataPoints": [
                            {
                                "timeUnixNano": now_ns,
                                "count": str(int(stats.get("count", 0))),
                                "sum": stats.get("sum", 0),
                                "quantileValues": [
                                    {"quantile": 0.5, "value": stats.get("p50", 0)},
                                    {"quantile": 0.95, "value": stats.get("p95", 0)},
                                    {"quantile": 0.99, "value": stats.get("p99", 0)},
                                ],
                            }
                        ],
                    },
                }
            )

        return {
            "resourceMetrics": [
                {
                    "resource": {
                        "attributes": [
                            {"key": "service.name", "value": {"stringValue": self._service_name}},
                        ],
                    },
                    "scopeMetrics": [
                        {
                            "scope": {"name": "loom.observability"},
                            "metrics": metrics,
                        }
                    ],
                }
            ],
        }

    def _send(self, payload: dict[str, Any]) -> None:
        try:
            data = json.dumps(payload).encode("utf-8")
            req = urllib.request.Request(
                self._endpoint,
                data=data,
                headers={
                    "Content-Type": "application/json",
                    **self._headers,
                },
                method="POST",
            )
            urllib.request.urlopen(req, timeout=self._timeout)
        except Exception:
            logger.debug("OTLP metrics export failed", exc_info=True)


class ConsoleSpanExporter(SpanExporter):
    """
    结构化 JSON 输出到 stderr（开发调试用）

    用法：
        tracer.add_exporter(ConsoleSpanExporter())
    """

    def __init__(self, stream: Any = None, pretty: bool = False):
        self._stream = stream or sys.stderr
        self._pretty = pretty

    def export(self, span: Span) -> None:
        data = span.to_dict()
        indent = 2 if self._pretty else None
        line = json.dumps(data, ensure_ascii=False, default=str, indent=indent)
        self._stream.write(line + "\n")
        self._stream.flush()


def _to_otlp_value(value: Any) -> dict[str, Any]:
    """Python 值 → OTLP AnyValue"""
    if isinstance(value, bool):
        return {"boolValue": value}
    if isinstance(value, int):
        return {"intValue": str(value)}
    if isinstance(value, float):
        return {"doubleValue": value}
    return {"stringValue": str(value)}



"""
检索结果数据模型
"""

from dataclasses import dataclass, field

from loom.providers.knowledge.rag.models.chunk import TextChunk
from loom.providers.knowledge.rag.models.entity import Entity
from loom.providers.knowledge.rag.models.relation import Relation


@dataclass
class RetrievalResult:
    """
    检索结果 - 封装完整的检索输出

    Attributes:
        chunks: 检索到的文本块列表
        entities: 相关实体列表
        relations: 相关关系列表
        scores: chunk_id -> 相关度分数的映射
    """

    chunks: list[TextChunk] = field(default_factory=list)
    entities: list[Entity] = field(default_factory=list)
    relations: list[Relation] = field(default_factory=list)
    scores: dict[str, float] = field(default_factory=dict)

    def is_empty(self) -> bool:
        """检查结果是否为空"""
        return len(self.chunks) == 0

    def get_top_chunks(self, n: int = 5) -> list[TextChunk]:
        """获取得分最高的 N 个文本块"""
        sorted_chunks = sorted(
            self.chunks,
            key=lambda c: self.scores.get(c.id, 0.0),
            reverse=True,
        )
        return sorted_chunks[:n]



"""
RAG 存储层

提供 Chunk、Entity、Relation 的存储接口和内存实现
"""

from loom.providers.knowledge.rag.stores.base import BaseStore
from loom.providers.knowledge.rag.stores.chunk_store import (
    ChunkStore,
    InMemoryChunkStore,
)
from loom.providers.knowledge.rag.stores.entity_store import (
    EntityStore,
    InMemoryEntityStore,
)
from loom.providers.knowledge.rag.stores.relation_store import (
    InMemoryRelationStore,
    RelationStore,
)

__all__ = [
    "BaseStore",
    "ChunkStore",
    "InMemoryChunkStore",
    "EntityStore",
    "InMemoryEntityStore",
    "RelationStore",
    "InMemoryRelationStore",
]



"""
内存知识库实现

简单的内存知识库，使用关键词匹配
"""

from typing import Any

from loom.providers.knowledge.base import KnowledgeBaseProvider, KnowledgeItem


class InMemoryKnowledgeBase(KnowledgeBaseProvider):
    """
    简单的内存知识库实现

    特点：
    - 使用关键词匹配
    - 数据存储在内存中
    - 适合开发和小规模部署
    """

    def __init__(self):
        self.items: dict[str, KnowledgeItem] = {}

    def add_item(self, item: KnowledgeItem) -> None:
        """
        添加知识条目

        Args:
            item: 知识条目
        """
        self.items[item.id] = item

    async def query(
        self,
        query: str,
        limit: int = 5,
        _filters: dict[str, Any] | None = None,
    ) -> list[KnowledgeItem]:
        """
        简单的关键词匹配查询

        Args:
            query: 查询文本
            limit: 返回结果数量
            _filters: 过滤条件（暂未使用）

        Returns:
            匹配的知识条目列表
        """
        query_lower = query.lower()
        matches = []

        for item in self.items.values():
            if query_lower in item.content.lower():
                matches.append(item)

        # 按匹配次数排序（简单的相关度计算）
        matches.sort(key=lambda x: x.content.lower().count(query_lower), reverse=True)

        return matches[:limit]

    async def get_by_id(self, knowledge_id: str) -> KnowledgeItem | None:
        """
        根据ID获取知识条目

        Args:
            knowledge_id: 知识条目ID

        Returns:
            知识条目，如果不存在则返回None
        """
        return self.items.get(knowledge_id)



"""
AgentFactory - Agent 工厂

从 Agent.core 提取的创建逻辑。
负责配置规范化、组件组装、参数合并。
"""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any
from uuid import uuid4

from loom.config.context import ContextConfig
from loom.config.memory import MemoryConfig
from loom.config.tool import ToolConfig
from loom.events.event_bus import EventBus
from loom.memory.compaction import CompactionConfig
from loom.runtime.session_lane import SessionIsolationMode
from loom.security.tool_policy import ToolPolicy
from loom.tools.skills.registry import skill_market

if TYPE_CHECKING:
    from loom.agent.core import Agent
    from loom.providers.llm.interface import LLMProvider


class AgentFactory:
    """
    Agent 工厂

    将 Agent.create() 的配置规范化逻辑提取为独立类：
    - EventBus 自动创建
    - ToolConfig / ContextConfig 规范化
    - Capabilities 提取
    - Skills 目录 / Loader 处理
    - SandboxToolManager 自动创建
    - node_id 生成
    """

    @staticmethod
    def create(
        llm: LLMProvider,
        *,
        system_prompt: str = "",
        tools: list[dict[str, Any]] | None = None,
        skills: list[str] | None = None,
        capabilities: Any | None = None,
        node_id: str | None = None,
        parent_node_id: str | None = None,
        agent_role: str | None = None,
        event_bus: Any | None = None,
        session: Any | None = None,
        knowledge_base: Any | None = None,
        max_context_tokens: int = 4000,
        max_iterations: int = 10,
        tool_policy: ToolPolicy | None = None,
        tool_config: ToolConfig | dict[str, Any] | None = None,
        context_config: ContextConfig | dict[str, Any] | None = None,
        skills_dir: str | Path | list[str | Path] | None = None,
        skill_loaders: list[Any] | None = None,
        memory_config: MemoryConfig | dict[str, Any] | None = None,
        context_budget_config: Any | None = None,
        session_isolation: SessionIsolationMode | str | None = None,
        compaction_config: CompactionConfig | dict[str, Any] | None = None,
        shared_pool: Any | None = None,
        embedding_provider: Any | None = None,
        **kwargs: Any,
    ) -> Agent:
        """
        创建 Agent 的推荐方式（一步到位）。

        v0.5.1: 支持渐进式披露的聚合配置

        Args:
            llm: LLM 提供者
            system_prompt: 系统提示词
            tools: 工具列表（简单用法，或使用 tool_config）
            skills: 技能 ID 列表（简单用法，或使用 tool_config）
            capabilities: CapabilityRegistry 实例（高级配置）
            node_id: 节点 ID（可选，默认自动生成）
            parent_node_id: 父节点ID（用于结构化命名）
            agent_role: Agent角色（用于SSE识别）
            event_bus: 事件总线（可选，未传入时自动创建）
            knowledge_base: 知识库提供者（可选）
            max_context_tokens: 最大上下文 token 数
            max_iterations: 最大迭代次数
            tool_policy: 工具权限策略（可选）
            tool_config: 工具聚合配置（ToolConfig 或 dict）
            context_config: 上下文聚合配置（ContextConfig 或 dict）
            **kwargs: 其他参数传递给 Agent.__init__

        Returns:
            Agent 实例
        """
        from loom.agent.core import Agent

        # 未传 event_bus 时自动创建
        if event_bus is None:
            event_bus = EventBus()

        # 处理 tool_config（聚合配置）
        tools, skills, skills_dir, skill_loaders = AgentFactory._normalize_tool_config(
            tool_config,
            tools,
            skills,
            skills_dir,
            skill_loaders,
        )

        # 处理 capabilities 参数（高级配置）
        AgentFactory._extract_capabilities(capabilities, kwargs)

        # 处理 Skills 目录 / Loader
        AgentFactory._setup_skill_loaders(skills_dir, skill_loaders, kwargs)

        # 处理 skills 参数（简单配置）
        AgentFactory._setup_skills(skills, kwargs)

        # 处理上下文配置（聚合入口）
        memory_config, context_budget_config, compaction_config, session_isolation = (
            AgentFactory._normalize_context_config(
                context_config,
                memory_config,
                context_budget_config,
                compaction_config,
                session_isolation,
            )
        )

        if session_isolation is None:
            session_isolation = SessionIsolationMode.STRICT

        # 显式参数合并到 kwargs
        if memory_config is not None:
            kwargs["memory_config"] = memory_config
        if context_budget_config is not None:
            kwargs["context_budget_config"] = context_budget_config
        if session_isolation is not None:
            kwargs["session_isolation"] = session_isolation
        if compaction_config is not None:
            kwargs["compaction_config"] = compaction_config
        if tool_policy is not None:
            kwargs["tool_policy"] = tool_policy
        if shared_pool is not None:
            kwargs["shared_pool"] = shared_pool
        if embedding_provider is not None:
            kwargs["embedding_provider"] = embedding_provider

        # 自动创建 SandboxToolManager（tools 含可调用对象时）
        tools = AgentFactory._setup_sandbox(tools, event_bus, kwargs)

        # 生成 node_id
        effective_node_id = AgentFactory._resolve_node_id(
            node_id,
            parent_node_id,
            agent_role,
        )

        return Agent(
            node_id=effective_node_id,
            llm_provider=llm,
            system_prompt=system_prompt,
            tools=tools,
            event_bus=event_bus,
            session=session,
            knowledge_base=knowledge_base,
            max_context_tokens=max_context_tokens,
            max_iterations=max_iterations,
            **kwargs,
        )

    # ==================== 内部规范化方法 ====================

    @staticmethod
    def _normalize_tool_config(
        tool_config: ToolConfig | dict[str, Any] | None,
        tools: list[dict[str, Any]] | None,
        skills: list[str] | None,
        skills_dir: str | Path | list[str | Path] | None,
        skill_loaders: list[Any] | None,
    ) -> tuple[
        list[dict[str, Any]] | None,
        list[str] | None,
        str | Path | list[str | Path] | None,
        list[Any] | None,
    ]:
        if tool_config is None:
            return tools, skills, skills_dir, skill_loaders

        if isinstance(tool_config, dict):
            tool_config = ToolConfig(**tool_config)
        elif not isinstance(tool_config, ToolConfig):
            raise TypeError("tool_config must be ToolConfig or dict")

        if tools is None and tool_config.tools is not None:
            tools = tool_config.tools
        if skills is None and tool_config.skills is not None:
            skills = tool_config.skills
        if skills_dir is None and tool_config.skills_dir is not None:
            skills_dir = tool_config.skills_dir
        if skill_loaders is None and tool_config.skill_loaders is not None:
            skill_loaders = tool_config.skill_loaders

        return tools, skills, skills_dir, skill_loaders

    @staticmethod
    def _extract_capabilities(capabilities: Any | None, kwargs: dict[str, Any]) -> None:
        if capabilities is None:
            return
        if "sandbox_manager" not in kwargs and hasattr(capabilities, "tool_manager"):
            kwargs["sandbox_manager"] = capabilities.tool_manager
        if "skill_registry" not in kwargs and hasattr(capabilities, "skill_registry"):
            kwargs["skill_registry"] = capabilities.skill_registry
        if "skill_activator" not in kwargs and hasattr(capabilities, "skill_activator"):
            kwargs["skill_activator"] = capabilities.skill_activator

    @staticmethod
    def _setup_skill_loaders(
        skills_dir: str | Path | list[str | Path] | None,
        skill_loaders: list[Any] | None,
        kwargs: dict[str, Any],
    ) -> None:
        if skills_dir is None and not skill_loaders:
            return

        from loom.tools.skills.filesystem_loader import FilesystemSkillLoader

        if "skill_registry" not in kwargs:
            kwargs["skill_registry"] = skill_market
        registry = kwargs["skill_registry"]

        if skills_dir is not None:
            dirs = skills_dir if isinstance(skills_dir, list | tuple | set) else [skills_dir]
            for skills_path in dirs:
                registry.register_loader(FilesystemSkillLoader(skills_path))

        if skill_loaders:
            for loader in skill_loaders:
                registry.register_loader(loader)

    @staticmethod
    def _setup_skills(skills: list[str] | None, kwargs: dict[str, Any]) -> None:
        if skills is None:
            return

        if "skill_registry" not in kwargs:
            kwargs["skill_registry"] = skill_market

        if "config" not in kwargs:
            from loom.config.agent import AgentConfig

            kwargs["config"] = AgentConfig(enabled_skills=set(skills))
        else:
            kwargs["config"].enabled_skills = set(skills)

    @staticmethod
    def _normalize_context_config(
        context_config: ContextConfig | dict[str, Any] | None,
        memory_config: MemoryConfig | dict[str, Any] | None,
        context_budget_config: Any | None,
        compaction_config: CompactionConfig | dict[str, Any] | None,
        session_isolation: SessionIsolationMode | str | None,
    ) -> tuple[Any, Any, Any, Any]:
        if context_config is None:
            return memory_config, context_budget_config, compaction_config, session_isolation

        if isinstance(context_config, dict):
            context_config = ContextConfig(**context_config)
        elif not isinstance(context_config, ContextConfig):
            raise TypeError("context_config must be ContextConfig or dict")

        if memory_config is None and context_config.memory is not None:
            memory_config = context_config.memory
        if context_budget_config is None and context_config.budget is not None:
            context_budget_config = context_config.budget
        if compaction_config is None and context_config.compaction is not None:
            compaction_config = context_config.compaction
        if session_isolation is None:
            session_isolation = context_config.session_isolation

        return memory_config, context_budget_config, compaction_config, session_isolation

    @staticmethod
    def _setup_sandbox(
        tools: list[Any] | None,
        event_bus: Any,
        kwargs: dict[str, Any],
    ) -> list[Any] | None:
        tools_list = tools or []
        tools_dicts = [t for t in tools_list if isinstance(t, dict)]
        tools_callables = [t for t in tools_list if callable(t)]
        sandbox_manager_in = kwargs.get("sandbox_manager")

        if tools_callables and sandbox_manager_in is None:
            import tempfile

            from loom.tools.core.sandbox import Sandbox
            from loom.tools.core.sandbox_manager import SandboxToolManager

            sandbox_path = tempfile.mkdtemp(prefix="loom_agent_")
            sandbox = Sandbox(sandbox_path, auto_create=True)
            manager = SandboxToolManager(sandbox, event_bus=event_bus)
            kwargs["sandbox_manager"] = manager
            kwargs["_pending_tool_callables"] = tools_callables
            tools = tools_dicts

        return tools

    @staticmethod
    def _resolve_node_id(
        node_id: str | None,
        parent_node_id: str | None,
        agent_role: str | None,
    ) -> str:
        if node_id is not None:
            return node_id
        if parent_node_id and agent_role:
            return f"{parent_node_id}:{agent_role}"
        if parent_node_id:
            return f"{parent_node_id}:{str(uuid4())[:8]}"
        if agent_role:
            return f"{agent_role}:{str(uuid4())[:8]}"
        return str(uuid4())
